@@ -1,5 +1,5 @@
 #!/bin/sh
-# $Id: g4_makedist.sh 21499 2007-12-19 16:50:00Z brun $
+# $Id: g4_makedist.sh 33605 2010-05-21 13:56:25Z brun $
 
 # Script to produce source and optionally binary distribution of geant4_vmc.
 # Called by main Makefile.
@@ -8,7 +8,7 @@
 # $ROOTSYS/build/unix/makedist.sh
 # Author: Fons Rademakers, 29/2/2000
 #
-# Usage: makedist.sh [gcc_version] [lib]
+# Usage: g4_makedist.sh [gcc_version] [lib]
 #
 # By I.Hrivnacova, 7/10/2002
 
@@ -39,23 +39,14 @@ else
 fi  
 TARFILE=geant4_vmc.$VERSION.$TYPE"tar"
 
-TAR=`which gtar`
-dum=`echo $TAR | grep "no gtar"`
-stat=$?
-if [ "$TAR" = '' ] || [ $stat = 0 ]; then
-   TAR="tar cvf"
-   rm -f $TARFILE.gz
-   EXCLUDE=
-else 
-   TAR=$TAR" zcvf"
-   rm -f $TARFILE.gz
-   TARFILE=$TARFILE".gz"
-   EXCLUDE="--exclude .svn"
-fi
+TAR="tar zcvf"
+rm -f $TARFILE.gz
+TARFILE=$TARFILE".gz"
+EXCLUDE="--exclude .svn"
 
-$TAR $TARFILE $EXCLUDE geant4_vmc/README geant4_vmc/LICENSE \
+$TAR $TARFILE $EXCLUDE geant4_vmc/README geant4_vmc/LICENSE geant4_vmc/Makefile \
    geant4_vmc/"history" geant4_vmc/Geant4VMC.html geant4_vmc/version_number  \
-   geant4_vmc/"source" geant4_vmc/examples $MAKELIB
+   geant4_vmc/g4root geant4_vmc/"source" geant4_vmc/examples $MAKELIB
 
 cd $CURDIR
 
