@@ -1,4 +1,4 @@
-/* @(#)root/html:$Id: LinkDef.h 20882 2007-11-19 11:31:26Z rdm $ */
+/* @(#)root/html:$Id: LinkDef.h 26828 2008-12-11 11:24:38Z axel $ */
 
 /*************************************************************************
  * Copyright (C) 1995-2000, Rene Brun and Fons Rademakers.               *
@@ -13,6 +13,7 @@
 #pragma link off all globals;
 #pragma link off all classes;
 #pragma link off all functions;
+#pragma link C++ nestedclasses;
 
 #pragma link C++ global gHtml;
 
@@ -27,4 +28,12 @@
 #pragma link C++ class TClassDocInfo;
 #pragma link C++ class TModuleDocInfo;
 #pragma link C++ class TLibraryDocInfo;
+#pragma link C++ class THtml::THelperBase;
+#pragma link C++ class THtml::TModuleDefinition;
+#pragma link C++ class THtml::TFileDefinition;
+#pragma link C++ class THtml::TPathDefinition;
+#pragma link C++ class THtml::TFileSysEntry;
+#pragma link C++ class THtml::TFileSysDir;
+#pragma link C++ class THtml::TFileSysRoot;
+#pragma link C++ class THtml::TFileSysDB;
 #endif
