@@ -1,4 +1,4 @@
-// @(#)root/test:$Id: Tetris.cxx 20882 2007-11-19 11:31:26Z rdm $
+// @(#)root/test:$Id: Tetris.cxx 38509 2011-03-18 14:54:32Z pcanal $
 // Author: Valeriy Onuchin & Fons Rademakers   04/10/98
 
 ///////////////////////////////////////////////////////////////////
@@ -663,7 +663,7 @@ void TetrisBoard::PieceDropped(TetrisPiece* piece, int height)
    fIsDropped = kFALSE;
 }
 
-void TetrisBoard::Print(const Text_t *) const
+void TetrisBoard::Print(const char *) const
 {
    // Used for testing
 
@@ -810,7 +810,7 @@ void InfoPad::PaintModified()
 
    char    str[40];
 
-   sprintf(str,"%d",fValue);
+   snprintf(str,40,"%d",fValue);
 
    TObject *obj = GetListOfPrimitives()->First();
 
