@@ -1,4 +1,4 @@
-// @(#)root/test:$Id: stress.cxx 20882 2007-11-19 11:31:26Z rdm $
+// @(#)root/test:$Id: stress.cxx 40518 2011-08-09 22:28:22Z pcanal $
 // Author: Rene Brun   05/11/98
 
 /////////////////////////////////////////////////////////////////
@@ -69,6 +69,7 @@
 //_____________________________batch only_____________________
 #ifndef __CINT__
 
+#include <stdlib.h>
 #include <TROOT.h>
 #include <TSystem.h>
 #include <TH1.h>
@@ -84,6 +85,7 @@
 #include <TRandom.h>
 #include <TPostScript.h>
 #include <TNtuple.h>
+#include <TTreeCache.h>
 #include <TChain.h>
 #include <TCut.h>
 #include <TCutG.h>
@@ -92,6 +94,7 @@
 #include <TSystem.h>
 #include <TApplication.h>
 #include <TClassTable.h>
+#include <Compression.h>
 #include "Event.h"
 
 void stress(Int_t nevent, Int_t style, Int_t printSubBenchmark, UInt_t portion );
@@ -186,11 +189,18 @@ void stress(Int_t nevent, Int_t style = 1,
    Bool_t UNIX = strcmp(gSystem->GetName(), "Unix") == 0;
    printf("******************************************************************\n");
    if (UNIX) {
-      FILE *fp = gSystem->OpenPipe("uname -a", "r");
-      char line[60];
-      fgets(line,60,fp); line[59] = 0;
-      printf("*  SYS: %s\n",line);
-      gSystem->ClosePipe(fp);
+      TString sp = gSystem->GetFromPipe("uname -a");
+      sp.Resize(60);
+      printf("*  SYS: %s\n",sp.Data());
+      if (strstr(gSystem->GetBuildNode(),"Linux")) {
+         sp = gSystem->GetFromPipe("lsb_release -d -s");
+         printf("*  SYS: %s\n",sp.Data());
+      }
+      if (strstr(gSystem->GetBuildNode(),"Darwin")) {
+         sp  = gSystem->GetFromPipe("sw_vers -productVersion");
+         sp += " Mac OS X ";
+         printf("*  SYS: %s\n",sp.Data());
+      }
    } else {
       const char *os = gSystem->Getenv("OS");
       if (!os) printf("*  SYS: Windows 95\n");
@@ -220,6 +230,8 @@ void stress(Int_t nevent, Int_t style = 1,
    printf("******************************************************************\n");
    printf("*  ROOTMARKS =%6.1f   *  Root%-8s  %d/%d\n",rootmarks,gROOT->GetVersion(),gROOT->GetVersionDate(),gROOT->GetVersionTime());
    printf("******************************************************************\n");
+   
+   delete gBenchmark;
 }
 
 //_______________________________________________________________
@@ -241,7 +253,7 @@ void Bprint(Int_t id, const char *title)
   // Print test program number and its title
    const Int_t kMAX = 65;
    char header[80];
-   sprintf(header,"Test %2d : %s",id,title);
+   snprintf(header,80,"Test %2d : %s",id,title);
    Int_t nch = strlen(header);
    for (Int_t i=nch;i<kMAX;i++) header[i] = '.';
    header[kMAX] = 0;
@@ -266,7 +278,7 @@ void stress1()
    f1form->SetParameters(f1params);
 
    //Create an histogram and fill it randomly with f1form
-   gRandom->SetSeed();
+   gRandom->SetSeed(65539);
    TH1F *h1form = new TH1F("h1form","distribution from f1form",100,-10,10);
    TH1F *h1diff = (TH1F*)h1form->Clone();
    h1diff->SetName("h1diff");
@@ -280,7 +292,7 @@ void stress1()
    f1->SetParameters(f1params);
 
    //Create an histogram and fill it randomly with f1int
-   gRandom->SetSeed(); //make sure we start with the same random numbers
+   gRandom->SetSeed(65539); //make sure we start with the same random numbers
    TH1F *h1int = new TH1F("h1int","distribution from f1int",100,-10,10);
    h1int->FillRandom("f1int",10000);
 
@@ -319,6 +331,9 @@ void stress1()
    h1int->Write();
    ntotout += local.GetBytesWritten();
    //do not close the file. should be done by the destructor automatically
+   delete h1int;
+   delete h1form;
+   delete h1diff;
 }
 
 //_______________________________________________________________
@@ -384,7 +399,7 @@ void stress4()
    f2form->SetParameters(f2params);
 
    //Create an histogram and fill it randomly with f2form
-   gRandom->SetSeed();
+   gRandom->SetSeed(65539);
    TH2F *h2form = new TH2F("h2form","distribution from f2form",40,-10,10,40,-10,10);
    Int_t nentries = 100000;
    h2form->FillRandom("f2form",nentries);
@@ -464,7 +479,7 @@ void stress5()
    FILE *fp = fopen("stress.ps","r");
    char line[260];
    Int_t nlines = 0;
-   Int_t nlinesGood = 675;
+   Int_t nlinesGood = 632;
    while (fgets(line,255,fp)) {
       nlines++;
    }
@@ -472,13 +487,15 @@ void stress5()
    ntotin  += f.GetBytesRead();
    ntotout += f.GetBytesWritten();
    Bool_t OK = kTRUE;
-   if (nlines < nlinesGood-40 || nlines > nlinesGood+40) OK = kFALSE;
+   if (nlines < nlinesGood-110 || nlines > nlinesGood+110) OK = kFALSE;
    if (OK) printf("OK\n");
    else    {
       printf("failed\n");
       printf("%-8s nlines in stress.ps file = %d\n"," ",nlines);
    }
+   delete c1;
    if (gPrintSubBench) { printf("Test  5 : "); gBenchmark->Show("stress");gBenchmark->Start("stress"); }
+   
 }
 
 //_______________________________________________________________
@@ -491,7 +508,7 @@ void stress6()
 
    TFile f("stress.root","update");
    // create a new subdirectory for each plane
-   gRandom->SetSeed();
+   gRandom->SetSeed(65539);
    const Int_t nplanes = 10;
    const Int_t ncounters = 100;
    char dirname[50];
@@ -503,17 +520,17 @@ void stress6()
    TH1F *hsumPlanes = new TH1F("hsumPlanes","Sum of all planes",100,0,100);
    //Create a subdirectory per detector plane
    for (i=0;i<nplanes;i++) {
-      sprintf(dirname,"plane%d",i);
+      snprintf(dirname,50,"plane%d",i);
       TDirectory *cdplane = f.mkdir(dirname);
       if (cdplane == 0) continue;
       cdplane->cd();
       // create counter histograms
       for (j=0;j<ncounters;j++) {
-         sprintf(hname,"h%d_%dN",i,j);
-         sprintf(htitle,"hist for counter:%d in plane:%d North",j,i);
+         snprintf(hname,20,"h%d_%dN",i,j);
+         snprintf(htitle,80,"hist for counter:%d in plane:%d North",j,i);
          hn[j] = new TH1S(hname,htitle,100,0,100);
-         sprintf(hname,"h%d_%dS",i,j);
-         sprintf(htitle,"hist for counter:%d in plane:%d South",j,i);
+         snprintf(hname,20,"h%d_%dS",i,j);
+         snprintf(htitle,80,"hist for counter:%d in plane:%d South",j,i);
          hs[j] = new TH1S(hname,htitle,100,0,100);
       }
       // fill counter histograms randomly
@@ -531,12 +548,12 @@ void stress6()
    // Now read back all objects from all subdirectories
    // Add North and south histograms in hsumPlanes
    for (i=0;i<nplanes;i++) {
-      sprintf(dirname,"plane%d",i);
+      snprintf(dirname,50,"plane%d",i);
       f.cd(dirname);
       for (j=0;j<ncounters;j++) {
-         sprintf(hname,"h%d_%dN",i,j);
+         snprintf(hname,20,"h%d_%dN",i,j);
          TH1S *hnorth; gDirectory->GetObject(hname,hnorth);
-         sprintf(hname,"h%d_%dS",i,j);
+         snprintf(hname,20,"h%d_%dS",i,j);
          TH1S *hsouth; gDirectory->GetObject(hname,hsouth);
          if (hnorth == 0 || hsouth == 0) continue;
          hsumPlanes->Add(hnorth);
@@ -579,7 +596,7 @@ void stress7()
 
    TFile f("stress.root","update");
    // Create and fill a TNtuple
-   gRandom->SetSeed();
+   gRandom->SetSeed(65539);
    TNtuple *ntuple = new TNtuple("ntuple","Demo ntuple","px:py:pz:random:i");
    Float_t px, py, pz;
    Int_t nall = 50000;
@@ -611,6 +628,7 @@ void stress7()
    Int_t npxpyGood = 27918;
    hpxpy->Write();
    cutg->Write();
+   delete cutg;
 
    // Fill a TEventList using the standard cut
    ntuple->Draw(">>elist","py<0 && pz>4 && random<0.5","goff");
@@ -662,8 +680,8 @@ void stress7()
    TEventList *el[10];
    TEventList *elistall = new TEventList("elistall","Sum of all cuts");
    for (i=0;i<10;i++) {
-      sprintf(elistname,">>elist%d",i);
-      sprintf(cutname,"i 10 == %d",i); cutname[1] ='%';
+      snprintf(elistname,20,">>elist%d",i);
+      snprintf(cutname,20,"i 10 == %d",i); cutname[1] ='%';
       ntuple->Draw(elistname,cutname,"goff");
       gDirectory->GetObject(&elistname[2],el[i]);
       el[i]->Write();
@@ -722,6 +740,12 @@ Int_t stress8read(Int_t nevent)
    tree->SetBranchAddress("event",&event);
    Int_t nentries = (Int_t)tree->GetEntries();
    Int_t nev = TMath::Max(nevent,nentries);
+   //activate the treeCache
+   Int_t cachesize = 10000000; //this is the default value: 10 MBytes
+   tree->SetCacheSize(cachesize);
+   TTreeCache::SetLearnEntries(1); //one entry is sufficient to learn
+   TTreeCache *tc = (TTreeCache*)hfile->GetCacheRead();
+   tc->SetEntryRange(0,nevent);
    Int_t nb = 0;
    for (Int_t ev = 0; ev < nev; ev++) {
       nb += tree->GetEntry(ev);        //read complete event in memory
@@ -781,12 +805,17 @@ void stress8(Int_t nevent)
   // First step: make sure the Event shared library exists
   // This test dynamic linking when running in interpreted mode
    if (!TClassTable::GetDict("Event")) {
-      Bool_t UNIX = strcmp(gSystem->GetName(), "Unix") == 0;
-      Int_t st1 = gSystem->Load("$(ROOTSYS)/test/libEvent");
+      Int_t st1 = -1;
+      if (gSystem->DynamicPathName("$ROOTSYS/test/libEvent",kTRUE)) {
+         st1 = gSystem->Load("$(ROOTSYS)/test/libEvent");
+      }
       if (st1 == -1) {
-         st1 = gSystem->Load("test/libEvent");
+         if (gSystem->DynamicPathName("test/libEvent",kTRUE)) {
+            st1 = gSystem->Load("test/libEvent");
+         }
          if (st1 == -1) {
             printf("===>stress8 will try to build the libEvent library\n");
+            Bool_t UNIX = strcmp(gSystem->GetName(), "Unix") == 0;
             if (UNIX) gSystem->Exec("(cd $ROOTSYS/test; make Event)");
             else      gSystem->Exec("(cd %ROOTSYS%\\test && nmake libEvent.dll)");
             st1 = gSystem->Load("$(ROOTSYS)/test/libEvent");
@@ -795,19 +824,19 @@ void stress8(Int_t nevent)
    }
 
    // Create the file not compressed, in no-split mode and read it back
-   gRandom->SetSeed();
+   gRandom->SetSeed(65539);
    Int_t nbw0 = stress8write(100,0,0);
    Int_t nbr0 = stress8read(0);
    Event::Reset();
 
    // Create the file compressed, in no-split mode and read it back
-   gRandom->SetSeed();
+   gRandom->SetSeed(65539);
    Int_t nbw1 = stress8write(100,1,0);
    Int_t nbr1 = stress8read(0);
    Event::Reset();
 
    // Create the file compressed, in split mode and read it back
-   gRandom->SetSeed();
+   gRandom->SetSeed(65539);
    Int_t nbw2 = stress8write(nevent,1,9);
    Int_t nbr2 = stress8read(0);
    Event::Reset();
@@ -850,7 +879,7 @@ void stress9tree(TTree *tree, Int_t realTestNum)
 {
 // Test selections via TreeFormula
 // tree is a TTree when called by stress9
-// tree is a TChain when called from stres11
+// tree is a TChain when called from stress11
 // This is a quite complex test checking the results of TTree::Draw
 // or TChain::Draw with an explicit loop on events.
 // Also a good test for the interpreter
@@ -1152,6 +1181,11 @@ void stress10()
    Bprint(10,"Create 10 files starting from Event.root");
 
    TFile *hfile = new TFile("Event.root");
+   if (hfile==0 || hfile->IsZombie()) {
+      delete hfile;
+      printf("failed\n");
+      return;
+   }
    TTree *tree; hfile->GetObject("T",tree);
 
    Event *event = 0;
@@ -1163,10 +1197,12 @@ void stress10()
    TFile *chfile[10];
    Int_t file;
    for (file=0;file<10;file++) {
-      sprintf(filename,"Event_%d.root",file);
+      snprintf(filename,20,"Event_%d.root",file);
       chfile[file] = new TFile(filename,"recreate");
+      if (file>=5) {
+         chfile[file]->SetCompressionAlgorithm(ROOT::kLZMA);
+      }
       chTree[file] = (TTree*)tree->CloneTree(0);
-      chTree[file]->SetBranchAddress("event",&event);
    }
 
    // Fill the small trees
@@ -1174,7 +1210,7 @@ void stress10()
    Int_t evmod, nbin=0, nbout=0;
    EventHeader *head;
    for (Int_t ev=0;ev<nev;ev++) {
-      nbin +=tree->GetEntry(ev);
+      nbin += tree->GetEntry(ev);
       head = event->GetHeader();
       evmod = head->GetEvtNum()%10;
       nbout += chTree[evmod]->Fill();
@@ -1187,7 +1223,6 @@ void stress10()
       chfile[file]->Write();
       delete chfile[file];
    }
-   event->ResetHistogramPointer(); // fH was deleted above!!
    delete event;
    delete hfile;
    Event::Reset();
@@ -1223,7 +1258,7 @@ void stress11()
    char filename[20];
    Int_t file;
    for (file=0;file<10;file++) {
-      sprintf(filename,"Event_%d.root",file);
+      snprintf(filename,20,"Event_%d.root",file);
       chain->Add(filename);
    }
 
@@ -1289,7 +1324,7 @@ void stress13()
    char filename[20];
    Int_t file;
    for (file=0;file<10;file++) {
-      sprintf(filename,"Event_%d.root",file);
+      snprintf(filename,20,"Event_%d.root",file);
       chain->Add(filename);
    }
 
@@ -1335,6 +1370,10 @@ void stress15()
    //Get old file, old tree and set top branch address
    //We want to copy only a few branches.
    TFile *oldfile = new TFile("Event.root");
+   if (oldfile->IsZombie()) {
+      printf("failed\n");
+      return;
+   }   
    TTree *oldtree; oldfile->GetObject("T",oldtree);
    Event *event   = 0;
    oldtree->SetBranchAddress("event",&event);
@@ -1373,12 +1412,17 @@ void stress15()
 
    // Open old reference file of stress9
    oldfile = new TFile("stress_test9.root");
+   if (oldfile->IsZombie()) {
+      printf("failed\n");
+      return;
+   }
    TH1F *bNtrack; oldfile->GetObject("bNtrack",bNtrack);
    TH1F *bHmean;  oldfile->GetObject("bHmean",bHmean);
    Int_t cNtrack = HistCompare(hNtrack,bNtrack);
    Int_t cHmean  = HistCompare(hHmean, bHmean);
    delete newfile;
    delete oldfile;
+   Event::Reset();
    gROOT->GetList()->Delete();
 
    Bool_t OK = kTRUE;
@@ -1416,7 +1460,7 @@ void stress16()
    TCanvas *c = new TCanvas("laten","latency simulation",700,600);
    gROOT->LoadClass("TPostScript","Postscript");
    TPostScript ps("stress_lhcb.ps",112);
-   gRandom->SetSeed();
+   gRandom->SetSeed(65539);
    TFile f("stress_lhcb.root", "recreate");
    TH1F *pipe = new TH1F("pipe", "free in pipeline", nbuf+1, -0.5, nbuf+0.5);
    pipe->SetLineColor(2);
@@ -1426,7 +1470,7 @@ void stress16()
    TProfile::Approximate();
    for (i = 0; i <= nlev; i++) {
       char s[64];
-      sprintf(s, "buf%d", i);
+      snprintf(s,64, "buf%d", i);
       hp[i] = new TProfile(s, "in buffers", 1000, 0,nstep, -1., 1000.);
       hp[i]->SetLineColor(2);
    }
@@ -1495,9 +1539,11 @@ void stress16()
    FILE *fp = fopen("stress_lhcb.ps","r");
    char line[260];
    Int_t nlines = 0;
-   Int_t nlinesGood = 2303;
+   Int_t nlinesGood = 2121;
+   Bool_t counting = kFALSE;
    while (fgets(line,255,fp)) {
-      nlines++;
+      if (counting) nlines++;
+      if (strstr(line,"%%EndProlog")) counting = kTRUE;
    }
    fclose(fp);
    delete c;
