@@ -1,4 +1,4 @@
-// @(#)root/test:$Id: stressVector.cxx 20882 2007-11-19 11:31:26Z rdm $
+// @(#)root/test:$Id: stressVector.cxx 39958 2011-06-25 13:57:57Z axel $
 // Author: Lorenzo Moneta   06/2005 
 ///////////////////////////////////////////////////////////////////////////////////
 //
@@ -192,7 +192,11 @@ double testAddition( const std::vector<V *> & dataV, TStopwatch & tim, double& t
     for (unsigned int j = i +1; j < n; ++j) {
       V & v2 = *(dataV[j]); 
       V v3 = v1 + v2;
-      tot += v3.E();
+      if (i % 1) {
+        tot += v3.E();
+      } else {
+        tot -= v3.E();
+      }
     }
   }
   tim.Stop();
@@ -210,7 +214,11 @@ double testScale( const std::vector<V *> & dataV, TStopwatch & tim, double& t,
     V  & v1 = *(dataV[i]); 
     // scale
     v1 = 2.0*v1;
-    tot += v1.E();
+    if (i % 1) {
+      tot += v1.E();
+    } else {
+      tot -= v1.E();
+    }
   }
   tim.Stop();
   print(tim,s);
@@ -247,7 +255,11 @@ double testDeltaR( const std::vector<V *> & dataV, TStopwatch & tim, double& t,
     for (unsigned int j = i +1; j < n; ++j) {
       V & v2 = *(dataV[j]); 
       double delta = VectorUtil::DeltaR(v1,v2);
-      tot += delta;
+      if (i % 1) {
+        tot += delta;
+      } else {
+        tot -= delta;
+      }
     }
   }
   tim.Stop();
@@ -520,7 +532,7 @@ int main(int argc,const char *argv[]) {
       s1=a.testAddition   (v1, t, t1, "Addition TLorentzVector      " );  
       s2=a.testAddition   (v2, t, t2, "Addition XYZTVector          "  ); 
       s3=a.testAddition   (v3, t, t3, "Addition PtEtaPhiEVector     " );       
-      a.check("Addition",s1,s2,s3);
+      a.check("Addition",s1,s2,s3,10);
 
 
       s1=a.testScale   (v1, t, t1, "Scale of TLorentzVector      " );  
@@ -531,12 +543,7 @@ int main(int argc,const char *argv[]) {
       s1=a.testDeltaR   (v1, t, t1,      "DeltaR   TLorentzVector      " );  
       s2=a.testDeltaR   (v2, t, t2,      "DeltaR   XYZTVector          " ); 
       s3=a.testDeltaR   (v3, t, t3,      "DeltaR   PtEtaPhiEVector     " ); 
-#ifdef WIN32
-      //windows is bad here 
       a.check("DeltaR",s1,s2,s3,10);      
-#else
-      a.check("DeltaR",s1,s2,s3);      
-#endif
 
 
       int n1, n2, n3; 
