@@ -1,4 +1,4 @@
-// @(#)root/test:$Id: vvector.cxx 20882 2007-11-19 11:31:26Z rdm $
+// @(#)root/test:$Id: vvector.cxx 38509 2011-03-18 14:54:32Z pcanal $
 // Author: Fons Rademakers and Eddy Offermann  Nov 2003
 
 //////////////////////////////////////////////////////////////////////////
@@ -94,7 +94,7 @@ void StatusPrint(Int_t id,const Char_t *title,Bool_t status)
   // Print test program number and its title
    const Int_t kMAX = 65;
    char header[80];
-   sprintf(header,"Test %2d : %s",id,title);
+   snprintf(header,80,"Test %2d : %s",id,title);
    Int_t nch = strlen(header);
    for (Int_t i=nch;i<kMAX;i++) header[i] = '.';
    header[kMAX] = 0;
