@@ -1,5 +1,5 @@
-// @(#)root/tmva $Id: MethodANNBase.cxx 21630 2008-01-10 19:40:44Z brun $
-// Author: Andreas Hoecker, Matt Jachowski
+// @(#)root/tmva $Id: MethodANNBase.cxx 38609 2011-03-24 16:06:32Z evt $
+// Author: Andreas Hoecker, Peter Speckmayer, Matt Jachowski, Jan Therhaag, Jiahang Zhong
 
 /**********************************************************************************
  * Project: TMVA - a Root-integrated toolkit for multivariate data analysis       *
@@ -12,11 +12,19 @@
  *      from background.                                                          *
  *                                                                                *
  * Authors (alphabetical):                                                        *
- *      Andreas Hoecker  <Andreas.Hocker@cern.ch> - CERN, Switzerland             *
- *      Matt Jachowski   <jachowski@stanford.edu> - Stanford University, USA      *
+ *      Krzysztof Danielowski <danielow@cern.ch>       - IFJ & AGH, Poland        *
+ *      Andreas Hoecker       <Andreas.Hocker@cern.ch> - CERN, Switzerland        *
+ *      Matt Jachowski        <jachowski@stanford.edu> - Stanford University, USA *
+ *      Kamil Kraszewski      <kalq@cern.ch>           - IFJ & UJ, Poland         *
+ *      Maciej Kruk           <mkruk@cern.ch>          - IFJ & AGH, Poland        *
+ *      Peter Speckmayer      <peter.speckmayer@cern.ch> - CERN, Switzerland      *
+ *      Joerg Stelzer         <stelzer@cern.ch>        - DESY, Germany            *
+ *      Jan Therhaag       <Jan.Therhaag@cern.ch>     - U of Bonn, Germany        *
+ *      Jiahang Zhong         <Jiahang.Zhong@cern.ch>  - Academia Sinica, Taipei  *
  *                                                                                *
- * Copyright (c) 2005:                                                            *
+ * Copyright (c) 2005-2011:                                                       *
  *      CERN, Switzerland                                                         *
+ *      U. of Bonn, Germany                                                       *
  *                                                                                *
  * Redistribution and use in source and binary forms, with or without             *
  * modification, are permitted according to the terms listed in LICENSE           *
@@ -24,18 +32,22 @@
  **********************************************************************************/
 
 //_______________________________________________________________________
-//                                                                      
-// Base class for all TMVA methods using artificial neural networks      
-//                                                                      
+//
+// Base class for all TMVA methods using artificial neural networks
+//
 //_______________________________________________________________________
 
-#include "TString.h"
 #include <vector>
+#include <cstdlib>
+#include <stdexcept>
+
+#include "TString.h"
 #include "TTree.h"
 #include "TDirectory.h"
 #include "Riostream.h"
 #include "TRandom3.h"
 #include "TH2F.h"
+#include "TH1.h"
 
 #include "TMVA/MethodBase.h"
 #include "TMVA/MethodANNBase.h"
@@ -52,25 +64,37 @@ using std::vector;
 ClassImp(TMVA::MethodANNBase)
 
 //______________________________________________________________________________
-TMVA::MethodANNBase::MethodANNBase( const TString& jobName, const TString& methodTitle, DataSet& theData, 
-                                    const TString& theOption, TDirectory* theTargetDir )
-   : TMVA::MethodBase( jobName, methodTitle, theData, theOption, theTargetDir )
+TMVA::MethodANNBase::MethodANNBase( const TString& jobName,
+                                    Types::EMVA methodType,
+                                    const TString& methodTitle,
+                                    DataSetInfo& theData,
+                                    const TString& theOption,
+                                    TDirectory* theTargetDir )
+   : TMVA::MethodBase( jobName, methodType, methodTitle, theData, theOption, theTargetDir )
+   , fEstimator(kMSE)
+   , fUseRegulator(kFALSE)
+   , fRandomSeed(0)
 {
    // standard constructor
    // Note: Right now it is an option to choose the neuron input function,
    // but only the input function "sum" leads to weight convergence --
-   // otherwise the weights go to nan and lead to an ABORT.   
+   // otherwise the weights go to nan and lead to an ABORT.
    InitANNBase();
 
    DeclareOptions();
 }
 
 //______________________________________________________________________________
-TMVA::MethodANNBase::MethodANNBase( DataSet & theData, 
-                                    const TString& theWeightFile, TDirectory* theTargetDir )
-   : TMVA::MethodBase( theData, theWeightFile, theTargetDir ) 
+TMVA::MethodANNBase::MethodANNBase( Types::EMVA methodType,
+                                    DataSetInfo& theData,
+                                    const TString& theWeightFile,
+                                    TDirectory* theTargetDir )
+   : TMVA::MethodBase( methodType, theData, theWeightFile, theTargetDir )
+   , fEstimator(kMSE)
+   , fUseRegulator(kFALSE)
+   , fRandomSeed(0)
 {
-   // construct the Method from the weight file 
+   // construct the Method from the weight file
    InitANNBase();
 
    DeclareOptions();
@@ -79,18 +103,25 @@ TMVA::MethodANNBase::MethodANNBase( DataSet & theData,
 //______________________________________________________________________________
 void TMVA::MethodANNBase::DeclareOptions()
 {
-   // define the options (their key words) that can be set in the option string 
+   // define the options (their key words) that can be set in the option string
    // here the options valid for ALL MVA methods are declared.
    // know options: NCycles=xx              :the number of training cycles
    //               Normalize=kTRUE,kFALSe  :if normalised in put variables should be used
    //               HiddenLayser="N-1,N-2"  :the specification of the hidden layers
    //               NeuronType=sigmoid,tanh,radial,linar  : the type of activation function
    //                                                       used at the neuronn
-   //                
+   //
+
+   DeclareOptionRef( fNcycles    = 500,       "NCycles",         "Number of training cycles" );
+   DeclareOptionRef( fLayerSpec  = "N,N-1",   "HiddenLayers",    "Specification of hidden layer architecture" );
+   DeclareOptionRef( fNeuronType = "sigmoid", "NeuronType",      "Neuron activation function type" );
+   DeclareOptionRef( fRandomSeed = 1, "RandomSeed", "Random seed for initial synapse weights (0 means unique seed for each run; default value '1')");
+
+   DeclareOptionRef(fEstimatorS="MSE", "EstimatorType",
+                    "MSE (Mean Square Estimator) for Gaussian Likelihood or CE(Cross-Entropy) for Bernoulli Likelihood" ); //zjh
+   AddPreDefVal(TString("MSE"));  //zjh
+   AddPreDefVal(TString("CE"));   //zjh
 
-   DeclareOptionRef( fNcycles    = 3000,      "NCycles",      "Number of training cycles" );
-   DeclareOptionRef( fLayerSpec  = "N-1,N-2", "HiddenLayers", "Specification of hidden layer architecture" );
-   DeclareOptionRef( fNeuronType = "tanh",    "NeuronType",   "Neuron activation function type" );
 
    TActivationChooser aChooser;
    vector<TString>* names = aChooser.GetAllActivationNames();
@@ -107,15 +138,17 @@ void TMVA::MethodANNBase::DeclareOptions()
    delete names;
 }
 
+
 //______________________________________________________________________________
 void TMVA::MethodANNBase::ProcessOptions()
 {
-   // decode the options in the option string
-
-   MethodBase::ProcessOptions();
-
+   // do nothing specific at this moment
+   if      ( DoRegression() || DoMulticlass())  fEstimatorS = "MSE";    //zjh
+   if      (fEstimatorS == "MSE" )  fEstimator = kMSE;   
+   else if (fEstimatorS == "CE")    fEstimator = kCE;      //zjh
    vector<Int_t>* layout = ParseLayoutString(fLayerSpec);
    BuildNetwork(layout);
+   delete layout;
 }
 
 //______________________________________________________________________________
@@ -124,13 +157,13 @@ vector<Int_t>* TMVA::MethodANNBase::ParseLayoutString(TString layerSpec)
    // parse layout specification string and return a vector, each entry
    // containing the number of neurons to go in each successive layer
    vector<Int_t>* layout = new vector<Int_t>();
-   layout->push_back(GetNvar());
+   layout->push_back((Int_t)GetNvar());
    while(layerSpec.Length()>0) {
       TString sToAdd="";
       if (layerSpec.First(',')<0) {
          sToAdd = layerSpec;
          layerSpec = "";
-      } 
+      }
       else {
          sToAdd = layerSpec(0,layerSpec.First(','));
          layerSpec = layerSpec(layerSpec.First(',')+1,layerSpec.Length());
@@ -140,8 +173,18 @@ vector<Int_t>* TMVA::MethodANNBase::ParseLayoutString(TString layerSpec)
       nNodes += atoi(sToAdd);
       layout->push_back(nNodes);
    }
-   layout->push_back(1);  // one output node
-  
+   if( DoRegression() )
+      layout->push_back( DataInfo().GetNTargets() );  // one output node for each target
+   else if( DoMulticlass() )
+      layout->push_back( DataInfo().GetNClasses() );  // one output node for each class
+   else
+      layout->push_back(1);  // one output node (for signal/background classification)
+
+   int n = 0;
+   for( std::vector<Int_t>::iterator it = layout->begin(); it != layout->end(); it++ ){
+      n++;
+   }
+
    return layout;
 }
 
@@ -152,21 +195,23 @@ void TMVA::MethodANNBase::InitANNBase()
    fNetwork         = NULL;
    frgen            = NULL;
    fActivation      = NULL;
+   fOutput          = NULL; //zjh
    fIdentity        = NULL;
    fInputCalculator = NULL;
    fSynapses        = NULL;
-   fEstimatorHistTrain = NULL;  
-   fEstimatorHistTest  = NULL;  
+   fEstimatorHistTrain = NULL;
+   fEstimatorHistTest  = NULL;
 
-   // MLP prefers normalised input variables
-   SetNormalised( kTRUE );
+   // reset monitorign histogram vectors
+   fEpochMonHistS.clear();
+   fEpochMonHistB.clear();
+   fEpochMonHistW.clear();
 
    // these will be set in BuildNetwork()
    fInputLayer = NULL;
-   fOutputNeuron = NULL;
+   fOutputNeurons.clear();
 
-   if (fgFIXED_SEED) frgen = new TRandom3(1);   // fix output for debugging
-   else              frgen = new TRandom3(0);   // seed = 0 means random seed
+   frgen = new TRandom3(fRandomSeed);
 
    fSynapses = new TObjArray();
 }
@@ -189,12 +234,12 @@ void TMVA::MethodANNBase::DeleteNetwork()
          layer = (TObjArray*)fNetwork->At(i);
          DeleteNetworkLayer(layer);
       }
-    
       delete fNetwork;
    }
 
    if (frgen != NULL)            delete frgen;
    if (fActivation != NULL)      delete fActivation;
+   if (fOutput != NULL)          delete fOutput;  //zjh
    if (fIdentity != NULL)        delete fIdentity;
    if (fInputCalculator != NULL) delete fInputCalculator;
    if (fSynapses != NULL)        delete fSynapses;
@@ -202,13 +247,14 @@ void TMVA::MethodANNBase::DeleteNetwork()
    fNetwork         = NULL;
    frgen            = NULL;
    fActivation      = NULL;
+   fOutput          = NULL; //zjh
    fIdentity        = NULL;
    fInputCalculator = NULL;
    fSynapses        = NULL;
 }
 
 //______________________________________________________________________________
-void TMVA::MethodANNBase::DeleteNetworkLayer(TObjArray*& layer)
+void TMVA::MethodANNBase::DeleteNetworkLayer( TObjArray*& layer )
 {
    // delete a network layer
    TNeuron* neuron;
@@ -222,12 +268,17 @@ void TMVA::MethodANNBase::DeleteNetworkLayer(TObjArray*& layer)
 }
 
 //______________________________________________________________________________
-void TMVA::MethodANNBase::BuildNetwork(vector<Int_t>* layout, vector<Double_t>* weights)
+void TMVA::MethodANNBase::BuildNetwork( vector<Int_t>* layout, vector<Double_t>* weights, Bool_t fromFile )
 {
    // build network given a layout (number of neurons in each layer)
    // and optional weights array
 
-   fLogger << kINFO << "Building Network" << Endl;
+   if (fEstimatorS == "MSE")  fEstimator = kMSE;    //zjh
+   else if (fEstimatorS == "CE")    fEstimator = kCE;      //zjh
+   else Log()<<kWARNING<<"fEstimator="<<fEstimator<<"\tfEstimatorS="<<fEstimatorS<<Endl;
+   if (fEstimator!=kMSE && fEstimator!=kCE) Log()<<kWARNING<<"Estimator type unspecified \t"<<Endl; //zjh
+
+   Log() << kINFO << "Building Network" << Endl;
 
    DeleteNetwork();
    InitANNBase();
@@ -236,23 +287,30 @@ void TMVA::MethodANNBase::BuildNetwork(vector<Int_t>* layout, vector<Double_t>*
    TActivationChooser aChooser;
    fActivation = aChooser.CreateActivation(fNeuronType);
    fIdentity   = aChooser.CreateActivation("linear");
+   if (fEstimator==kMSE)  fOutput = aChooser.CreateActivation("linear");  //zjh
+   else if (fEstimator==kCE)   fOutput = aChooser.CreateActivation("sigmoid"); //zjh
    TNeuronInputChooser iChooser;
    fInputCalculator = iChooser.CreateNeuronInput(fNeuronInputType);
 
    fNetwork = new TObjArray();
-   BuildLayers(layout);
+   fRegulatorIdx.clear();		//zjh
+   fRegulators.clear();			//zjh
+   BuildLayers( layout, fromFile );
 
    // cache input layer and output neuron for fast access
    fInputLayer   = (TObjArray*)fNetwork->At(0);
    TObjArray* outputLayer = (TObjArray*)fNetwork->At(fNetwork->GetEntriesFast()-1);
-   fOutputNeuron = (TNeuron*)outputLayer->At(0);
+   fOutputNeurons.clear();
+   for (Int_t i = 0; i < outputLayer->GetEntries(); i++) {
+      fOutputNeurons.push_back( (TNeuron*)outputLayer->At(i) );
+   }
 
    if (weights == NULL) InitWeights();
    else                 ForceWeights(weights);
 }
 
 //______________________________________________________________________________
-void TMVA::MethodANNBase::BuildLayers(vector<Int_t>* layout)
+void TMVA::MethodANNBase::BuildLayers( vector<Int_t>* layout, Bool_t fromFile )
 {
    // build the network layers
 
@@ -263,66 +321,79 @@ void TMVA::MethodANNBase::BuildLayers(vector<Int_t>* layout)
 
    for (Int_t i = 0; i < numLayers; i++) {
       curLayer = new TObjArray();
-      BuildLayer(layout->at(i), curLayer, prevLayer, i, numLayers);
+      BuildLayer(layout->at(i), curLayer, prevLayer, i, numLayers, fromFile);
       prevLayer = curLayer;
       fNetwork->Add(curLayer);
    }
 
    // cache pointers to synapses for fast access, the order matters
-   for (Int_t i = 0; i < numLayers; i++) {                                       
-      TObjArray* layer = (TObjArray*)fNetwork->At(i);                             
-      Int_t numNeurons = layer->GetEntriesFast();                                 
-      for (Int_t j = 0; j < numNeurons; j++) {                                    
-         TNeuron* neuron = (TNeuron*)layer->At(j);                                 
-         Int_t numSynapses = neuron->NumPostLinks();                               
-         for (Int_t k = 0; k < numSynapses; k++) {                                 
-            TSynapse* synapse = neuron->PostLinkAt(k);                              
-            fSynapses->Add(synapse);                                   
-         }                                                                         
-      }                                                                           
-   }  
+   for (Int_t i = 0; i < numLayers; i++) {
+      TObjArray* layer = (TObjArray*)fNetwork->At(i);
+      Int_t numNeurons = layer->GetEntriesFast();
+      if (i!=0 && i!=numLayers-1) fRegulators.push_back(0.);  //zjh
+      for (Int_t j = 0; j < numNeurons; j++) {
+         if (i==0) fRegulators.push_back(0.);			//zjh
+         TNeuron* neuron = (TNeuron*)layer->At(j);
+         Int_t numSynapses = neuron->NumPostLinks();
+         for (Int_t k = 0; k < numSynapses; k++) {
+            TSynapse* synapse = neuron->PostLinkAt(k);
+            fSynapses->Add(synapse);
+            fRegulatorIdx.push_back(fRegulators.size()-1);	//zjh
+         }
+      }
+   }
 }
 
 //______________________________________________________________________________
-void TMVA::MethodANNBase::BuildLayer(Int_t numNeurons, TObjArray* curLayer, 
-                                     TObjArray* prevLayer, Int_t layerIndex, 
-                                     Int_t numLayers)
+void TMVA::MethodANNBase::BuildLayer( Int_t numNeurons, TObjArray* curLayer, 
+                                      TObjArray* prevLayer, Int_t layerIndex, 
+                                      Int_t numLayers, Bool_t fromFile )
 {
    // build a single layer with neurons and synapses connecting this
    // layer to the previous layer
 
    TNeuron* neuron;
    for (Int_t j = 0; j < numNeurons; j++) {
-      neuron = new TNeuron();
-      neuron->SetInputCalculator(fInputCalculator);
-
-      // input layer
-      if (layerIndex == 0) {
+      if (fromFile && (layerIndex != numLayers-1) && (j==numNeurons-1)){
+         neuron = new TNeuron();
          neuron->SetActivationEqn(fIdentity);
-         neuron->SetInputNeuron();
+         neuron->SetBiasNeuron();
+         neuron->ForceValue(1.0);
+         curLayer->Add(neuron);  
       }
       else {
-         // output layer
-         if (layerIndex == numLayers-1) {
-            neuron->SetOutputNeuron();
+         neuron = new TNeuron();
+         neuron->SetInputCalculator(fInputCalculator);
+         
+         // input layer
+         if (layerIndex == 0) {
             neuron->SetActivationEqn(fIdentity);
+            neuron->SetInputNeuron();
          }
-         // hidden layers
-         else neuron->SetActivationEqn(fActivation);
-
-         AddPreLinks(neuron, prevLayer);
+         else {
+            // output layer
+            if (layerIndex == numLayers-1) {
+               neuron->SetOutputNeuron();
+               neuron->SetActivationEqn(fOutput);     //zjh
+         }
+            // hidden layers
+            else neuron->SetActivationEqn(fActivation);
+            AddPreLinks(neuron, prevLayer);
+         }
+         
+         curLayer->Add(neuron);
       }
-
-      curLayer->Add(neuron);
    }
 
    // add bias neutron (except to output layer)
-   if (layerIndex != numLayers-1) {
-      neuron = new TNeuron();
-      neuron->SetActivationEqn(fIdentity);
-      neuron->SetBiasNeuron();
-      neuron->ForceValue(1.0);
-      curLayer->Add(neuron);
+   if(!fromFile){
+      if (layerIndex != numLayers-1) {
+         neuron = new TNeuron();
+         neuron->SetActivationEqn(fIdentity);
+         neuron->SetBiasNeuron();
+         neuron->ForceValue(1.0);
+         curLayer->Add(neuron);
+      }
    }
 }
 
@@ -375,7 +446,7 @@ void TMVA::MethodANNBase::ForceWeights(vector<Double_t>* weights)
 }
 
 //______________________________________________________________________________
-void TMVA::MethodANNBase::ForceNetworkInputs(Int_t ignoreIndex)
+void TMVA::MethodANNBase::ForceNetworkInputs( const Event* ev, Int_t ignoreIndex)
 {
    // force the input values of the input neurons
    // force the value for each input neuron
@@ -383,9 +454,10 @@ void TMVA::MethodANNBase::ForceNetworkInputs(Int_t ignoreIndex)
    Double_t x;
    TNeuron* neuron;
 
-   for (Int_t j = 0; j < GetNvar(); j++) {
-      if (j == ignoreIndex) x = 0;
-      else                  x = GetEventVal(j);
+   //   const Event* ev = GetEvent();
+   for (UInt_t j = 0; j < GetNvar(); j++) {
+
+      x = (j != (UInt_t)ignoreIndex)?ev->GetValue(j):0;
 
       neuron = GetInputNeuron(j);
       neuron->ForceValue(x);
@@ -410,6 +482,7 @@ void TMVA::MethodANNBase::ForceNetworkCalculations()
          neuron = (TNeuron*) curLayer->At(j);
          neuron->CalculateValue();
          neuron->CalculateActivationValue();
+
       }
    }
 }
@@ -418,7 +491,7 @@ void TMVA::MethodANNBase::ForceNetworkCalculations()
 void TMVA::MethodANNBase::PrintMessage(TString message, Bool_t force) const
 {
    // print messages, turn off printing by setting verbose and debug flag appropriately
-   if (Verbose() || Debug() || force) fLogger << kINFO << message << Endl;
+   if (Verbose() || Debug() || force) Log() << kINFO << message << Endl;
 }
 
 //______________________________________________________________________________
@@ -426,7 +499,7 @@ void TMVA::MethodANNBase::WaitForKeyboard()
 {
    // wait for keyboard input, for debugging
    string dummy;
-   fLogger << kINFO << "*** Type anything to continue (q to quit): ";
+   Log() << kINFO << "***Type anything to continue (q to quit): ";
    getline(cin, dummy);
    if (dummy == "q" || dummy == "Q") {
       PrintMessage( "quit" );
@@ -436,15 +509,14 @@ void TMVA::MethodANNBase::WaitForKeyboard()
 }
 
 //______________________________________________________________________________
-void TMVA::MethodANNBase::PrintNetwork()
+void TMVA::MethodANNBase::PrintNetwork() const
 {
    // print network representation, for debugging
-
    if (!Debug()) return;
 
-   fLogger << Endl;
+   Log() << kINFO << Endl;
    PrintMessage( "Printing network " );
-   fLogger << kINFO << "-------------------------------------------------------------------" << Endl;
+   Log() << kINFO << "-------------------------------------------------------------------" << Endl;
 
    TObjArray* curLayer;
    Int_t numLayers = fNetwork->GetEntriesFast();
@@ -454,13 +526,13 @@ void TMVA::MethodANNBase::PrintNetwork()
       curLayer = (TObjArray*)fNetwork->At(i);
       Int_t numNeurons = curLayer->GetEntriesFast();
 
-      fLogger << kINFO << "Layer #" << i << " (" << numNeurons << " neurons):" << Endl;
+      Log() << kINFO << "Layer #" << i << " (" << numNeurons << " neurons):" << Endl;
       PrintLayer( curLayer );
    }
 }
 
 //______________________________________________________________________________
-void TMVA::MethodANNBase::PrintLayer(TObjArray* layer)
+void TMVA::MethodANNBase::PrintLayer(TObjArray* layer) const
 {
    // print a single layer, for debugging
 
@@ -469,39 +541,41 @@ void TMVA::MethodANNBase::PrintLayer(TObjArray* layer)
   
    for (Int_t j = 0; j < numNeurons; j++) {
       neuron = (TNeuron*) layer->At(j);
-      fLogger << kINFO << "\tNeuron #" << j << " (LinksIn: " << neuron->NumPreLinks() 
+      Log() << kINFO << "\tNeuron #" << j << " (LinksIn: " << neuron->NumPreLinks() 
               << " , LinksOut: " << neuron->NumPostLinks() << ")" << Endl;
       PrintNeuron( neuron );
    }
 }
 
 //______________________________________________________________________________
-void TMVA::MethodANNBase::PrintNeuron(TNeuron* neuron)
+void TMVA::MethodANNBase::PrintNeuron(TNeuron* neuron) const
 {
    // print a neuron, for debugging
-   fLogger << kINFO 
+   Log() << kINFO 
            << "\t\tValue:\t"     << neuron->GetValue()
            << "\t\tActivation: " << neuron->GetActivationValue()
            << "\t\tDelta: "      << neuron->GetDelta() << Endl;
-   fLogger << kINFO << "\t\tActivationEquation:\t";
+   Log() << kINFO << "\t\tActivationEquation:\t";
    neuron->PrintActivationEqn();
-   fLogger << kINFO << "\t\tLinksIn:" << Endl;
+   Log() << kINFO << "\t\tLinksIn:" << Endl;
    neuron->PrintPreLinks();
-   fLogger << kINFO << "\t\tLinksOut:" << Endl;
+   Log() << kINFO << "\t\tLinksOut:" << Endl;
    neuron->PrintPostLinks();
 }
 
 //_______________________________________________________________________
-Double_t TMVA::MethodANNBase::GetMvaValue()
+Double_t TMVA::MethodANNBase::GetMvaValue( Double_t* err, Double_t* errUpper )
 {
    // get the mva value generated by the NN
    TNeuron* neuron;
 
    TObjArray* inputLayer = (TObjArray*)fNetwork->At(0);
 
-   for (Int_t i = 0; i < GetNvar(); i++) {
+   const Event * ev = GetEvent();
+
+   for (UInt_t i = 0; i < GetNvar(); i++) {
       neuron = (TNeuron*)inputLayer->At(i);
-      neuron->ForceValue( GetEventVal(i) );
+      neuron->ForceValue( ev->GetValue(i) );
    }
    ForceNetworkCalculations();
 
@@ -509,31 +583,267 @@ Double_t TMVA::MethodANNBase::GetMvaValue()
    TObjArray* outputLayer = (TObjArray*)fNetwork->At( fNetwork->GetEntriesFast()-1 );
    neuron = (TNeuron*)outputLayer->At(0);
 
+   // cannot determine error
+   NoErrorCalc(err, errUpper);
+
    return neuron->GetActivationValue();
 }
 
 //_______________________________________________________________________
-void TMVA::MethodANNBase::WriteWeightsToStream( ostream & o) const
+const std::vector<Float_t> &TMVA::MethodANNBase::GetRegressionValues() 
 {
-   // write the weights stream
+   // get the regression value generated by the NN
+   TNeuron* neuron;
+
+   TObjArray* inputLayer = (TObjArray*)fNetwork->At(0);
+
+   const Event * ev = GetEvent();
+
+   for (UInt_t i = 0; i < GetNvar(); i++) {
+      neuron = (TNeuron*)inputLayer->At(i);
+      neuron->ForceValue( ev->GetValue(i) );
+   }
+   ForceNetworkCalculations();
+
+   // check the output of the network
+   TObjArray* outputLayer = (TObjArray*)fNetwork->At( fNetwork->GetEntriesFast()-1 );
+
+   if (fRegressionReturnVal == NULL) fRegressionReturnVal = new std::vector<Float_t>();
+   fRegressionReturnVal->clear();
+
+   Event * evT = new Event(*ev);
+   UInt_t ntgts = outputLayer->GetEntriesFast();
+   for (UInt_t itgt = 0; itgt < ntgts; itgt++) {
+      evT->SetTarget(itgt,((TNeuron*)outputLayer->At(itgt))->GetActivationValue());
+   }
+
+   const Event* evT2 = GetTransformationHandler().InverseTransform( evT );
+   for (UInt_t itgt = 0; itgt < ntgts; itgt++) {
+      fRegressionReturnVal->push_back( evT2->GetTarget(itgt) );
+   }
+
+   delete evT;
+
+   return *fRegressionReturnVal;
+}
+
+//_______________________________________________________________________
+const std::vector<Float_t> &TMVA::MethodANNBase::GetMulticlassValues()
+{
+   // get the multiclass classification values generated by the NN
+   TNeuron* neuron;
+
+   TObjArray* inputLayer = (TObjArray*)fNetwork->At(0);
+
+   const Event * ev = GetEvent();
+   
+   for (UInt_t i = 0; i < GetNvar(); i++) {
+      neuron = (TNeuron*)inputLayer->At(i);
+      neuron->ForceValue( ev->GetValue(i) );
+   }
+   ForceNetworkCalculations();
+
+   // check the output of the network
+ 
+   if (fMulticlassReturnVal == NULL) fMulticlassReturnVal = new std::vector<Float_t>();
+   fMulticlassReturnVal->clear();
+   std::vector<Float_t> temp;
+
+   UInt_t nClasses = DataInfo().GetNClasses();
+   for (UInt_t icls = 0; icls < nClasses; icls++) {
+      temp.push_back(GetOutputNeuron( icls )->GetActivationValue() );
+   }
+   
+   for(UInt_t iClass=0; iClass<nClasses; iClass++){
+      Double_t norm = 0.0;
+      for(UInt_t j=0;j<nClasses;j++){
+         if(iClass!=j)
+            norm+=exp(temp[j]-temp[iClass]);
+         }
+      (*fMulticlassReturnVal).push_back(1.0/(1.0+norm));
+   }
+   
+   return *fMulticlassReturnVal;
+}
+
+
+//_______________________________________________________________________
+void TMVA::MethodANNBase::AddWeightsXMLTo( void* parent ) const 
+{
+   // create XML description of ANN classifier
    Int_t numLayers = fNetwork->GetEntriesFast();
-   o << "Weights" << endl;
+   void* wght = gTools().xmlengine().NewChild(parent, 0, "Weights");
+   void* xmlLayout = gTools().xmlengine().NewChild(wght, 0, "Layout");
+   gTools().xmlengine().NewAttr(xmlLayout, 0, "NLayers", gTools().StringFromInt(fNetwork->GetEntriesFast()) );
+   TString weights = "";
    for (Int_t i = 0; i < numLayers; i++) {
       TObjArray* layer = (TObjArray*)fNetwork->At(i);
       Int_t numNeurons = layer->GetEntriesFast();
+      void* layerxml = gTools().xmlengine().NewChild(xmlLayout, 0, "Layer");
+      gTools().xmlengine().NewAttr(layerxml, 0, "Index",    gTools().StringFromInt(i) );
+      gTools().xmlengine().NewAttr(layerxml, 0, "NNeurons", gTools().StringFromInt(numNeurons) );
       for (Int_t j = 0; j < numNeurons; j++) {
          TNeuron* neuron = (TNeuron*)layer->At(j);
          Int_t numSynapses = neuron->NumPostLinks();
+         void* neuronxml = gTools().AddChild(layerxml, "Neuron");
+         gTools().AddAttr(neuronxml, "NSynapses", gTools().StringFromInt(numSynapses) );
+         if(numSynapses==0) continue;
+         stringstream s("");
+         s.precision( 16 );
          for (Int_t k = 0; k < numSynapses; k++) {
             TSynapse* synapse = neuron->PostLinkAt(k);
-            o << "(layer" << i << ",neuron" << j << ")-(layer" 
-              << i+1 << ",neuron" << k << "): " 
-              << synapse->GetWeight() << endl;
+            s << std::scientific << synapse->GetWeight() << " ";
          }
+         gTools().AddRawLine( neuronxml, s.str().c_str() );
       }
    }
+
+   // if inverse hessian exists, write inverse hessian to weight file
+   if( fInvHessian.GetNcols()>0 ){
+      void* xmlInvHessian = gTools().xmlengine().NewChild(wght, 0, "InverseHessian");
+
+      // get the matrix dimensions
+      Int_t nElements = fInvHessian.GetNoElements();
+      Int_t nRows     = fInvHessian.GetNrows();
+      Int_t nCols     = fInvHessian.GetNcols();
+      gTools().xmlengine().NewAttr(xmlInvHessian, 0, "NElements", gTools().StringFromInt(nElements) );
+      gTools().xmlengine().NewAttr(xmlInvHessian, 0, "NRows", gTools().StringFromInt(nRows) );
+      gTools().xmlengine().NewAttr(xmlInvHessian, 0, "NCols", gTools().StringFromInt(nCols) );
+
+      // read in the matrix elements
+      Double_t* elements = new Double_t[nElements+10];
+      fInvHessian.GetMatrix2Array( elements );
+
+      // store the matrix elements row-wise
+      Int_t index = 0;
+      for( Int_t row = 0; row < nRows; ++row ){
+         void* xmlRow = gTools().xmlengine().NewChild(xmlInvHessian, 0, "Row");
+         gTools().xmlengine().NewAttr(xmlRow, 0, "Index", gTools().StringFromInt(row) );
+
+         // create the rows
+         stringstream s("");
+         s.precision( 16 );
+         for( Int_t col = 0; col < nCols; ++col ){
+            s << std::scientific << (*(elements+index)) << " ";
+            ++index;
+         }
+         gTools().xmlengine().AddRawLine( xmlRow, s.str().c_str() );
+      }
+      delete[] elements;
+   }
 }
 
+
+//_______________________________________________________________________
+void TMVA::MethodANNBase::ReadWeightsFromXML( void* wghtnode )
+{
+   // read MLP from xml weight file
+
+   // build the layout first
+   Bool_t fromFile = kTRUE;
+   vector<Int_t>* layout = new vector<Int_t>();
+
+   void* xmlLayout = NULL;
+   xmlLayout = gTools().GetChild(wghtnode, "Layout");
+   if( !xmlLayout )
+      xmlLayout = wghtnode;
+
+   UInt_t nLayers;
+   gTools().ReadAttr( xmlLayout, "NLayers", nLayers );
+   layout->resize( nLayers );
+
+   void* ch = gTools().xmlengine().GetChild(xmlLayout);
+   UInt_t index;
+   UInt_t nNeurons;
+   while (ch) {
+      gTools().ReadAttr( ch, "Index",   index   );
+      gTools().ReadAttr( ch, "NNeurons", nNeurons );
+      layout->at(index) = nNeurons;
+      ch = gTools().GetNextChild(ch);
+   }
+
+   BuildNetwork( layout, NULL, fromFile );
+   // fill the weights of the synapses
+   UInt_t nSyn;
+   Float_t weight;
+   ch = gTools().xmlengine().GetChild(xmlLayout);
+   UInt_t iLayer = 0;
+   while (ch) {  // layers
+      TObjArray* layer = (TObjArray*)fNetwork->At(iLayer);
+      gTools().ReadAttr( ch, "Index",   index   );
+      gTools().ReadAttr( ch, "NNeurons", nNeurons );
+
+      void* nodeN = gTools().GetChild(ch);
+      UInt_t iNeuron = 0;
+      while( nodeN ){ // neurons
+         TNeuron *neuron = (TNeuron*)layer->At(iNeuron);
+         gTools().ReadAttr( nodeN, "NSynapses", nSyn );
+         if( nSyn > 0 ){
+            const char* content = gTools().GetContent(nodeN);
+            std::stringstream s(content);
+            for (UInt_t iSyn = 0; iSyn<nSyn; iSyn++) { // synapses
+
+               TSynapse* synapse = neuron->PostLinkAt(iSyn);
+               s >> weight;
+               //Log() << kWARNING << neuron << " " << weight <<  Endl;
+               synapse->SetWeight(weight);
+            }
+         }
+         nodeN = gTools().GetNextChild(nodeN);
+         iNeuron++;
+      }
+      ch = gTools().GetNextChild(ch);
+      iLayer++;
+   }
+
+   delete layout;
+
+   void* xmlInvHessian = NULL;
+   xmlInvHessian = gTools().GetChild(wghtnode, "InverseHessian");
+   if( !xmlInvHessian )
+      // no inverse hessian available
+      return;
+
+   fUseRegulator = kTRUE;
+
+   Int_t nElements = 0;
+   Int_t nRows     = 0;
+   Int_t nCols     = 0;
+   gTools().ReadAttr( xmlInvHessian, "NElements", nElements );
+   gTools().ReadAttr( xmlInvHessian, "NRows", nRows );
+   gTools().ReadAttr( xmlInvHessian, "NCols", nCols );
+
+   // adjust the matrix dimensions
+   fInvHessian.ResizeTo( nRows, nCols );
+
+   // prepare an array to read in the values
+   Double_t* elements = new Double_t[nElements+10];
+
+
+
+   void* xmlRow = gTools().xmlengine().GetChild(xmlInvHessian);
+   Int_t row = 0;
+   index = 0;
+   while (xmlRow) {  // rows
+      gTools().ReadAttr( xmlRow, "Index",   row   );
+
+      const char* content = gTools().xmlengine().GetNodeContent(xmlRow);
+
+      std::stringstream s(content);
+      for (Int_t iCol = 0; iCol<nCols; iCol++) { // columns
+	 s >> (*(elements+index));
+	 ++index;
+      }
+      xmlRow = gTools().xmlengine().GetNext(xmlRow);
+      ++row;
+   }
+
+   fInvHessian.SetMatrixArray( elements );
+
+   delete[] elements;
+}
+
+
 //_______________________________________________________________________
 void TMVA::MethodANNBase::ReadWeightsFromStream( istream & istr)
 {
@@ -551,6 +861,7 @@ void TMVA::MethodANNBase::ReadWeightsFromStream( istream & istr)
 
    ForceWeights(weights);
    
+
    delete weights;
 }
 
@@ -567,12 +878,12 @@ const TMVA::Ranking* TMVA::MethodANNBase::CreateRanking()
    Double_t  importance, avgVal;
    TString varName;
 
-   for (Int_t i = 0; i < GetNvar(); i++) {
+   for (UInt_t ivar = 0; ivar < GetNvar(); ivar++) {
 
-      neuron = GetInputNeuron(i);
+      neuron = GetInputNeuron(ivar);
       Int_t numSynapses = neuron->NumPostLinks();
       importance = 0;
-      varName = GetInputVar(i); // fix this line
+      varName = GetInputVar(ivar); // fix this line
 
       // figure out average value of variable i
       Double_t meanS, meanB, rmsS, rmsB, xmin, xmax;
@@ -580,7 +891,7 @@ const TMVA::Ranking* TMVA::MethodANNBase::CreateRanking()
                   meanS, meanB, rmsS, rmsB, xmin, xmax );
 
       avgVal = (meanS + meanB) / 2.0; // change this into a real weighted average
-      if (IsNormalised()) avgVal = 0.5*(1 + Tools::NormVariable( avgVal, GetXmin( i ), GetXmax( i )));
+      if (IsNormalised()) avgVal = 0.5*(1 + gTools().NormVariable( avgVal, GetXmin( ivar ), GetXmax( ivar )));
 
       for (Int_t j = 0; j < numSynapses; j++) {
          synapse = neuron->PostLinkAt(j);
@@ -589,26 +900,19 @@ const TMVA::Ranking* TMVA::MethodANNBase::CreateRanking()
       
       importance *= avgVal * avgVal;
 
-      fRanking->AddRank( *new Rank( varName, importance ) );
+      fRanking->AddRank( Rank( varName, importance ) );
    }
 
    return fRanking;
 }
 
 //_______________________________________________________________________
-void TMVA::MethodANNBase::WriteMonitoringHistosToFile() const
+void TMVA::MethodANNBase::CreateWeightMonitoringHists( const TString& bulkname, 
+                                                       std::vector<TH1*>* hv ) const
 {
-   // write histograms to file
-   PrintMessage(Form("Write special histos to file: %s", BaseDir()->GetPath()), kTRUE);
-
-   //   BaseDir()->mkdir(GetName()+GetMethodName())->cd(); 
-
-   TH2F*      hist;
+   TH2F* hist;
    Int_t numLayers = fNetwork->GetEntriesFast();
 
-   if (fEstimatorHistTrain) fEstimatorHistTrain->Write();
-   if (fEstimatorHistTest ) fEstimatorHistTest ->Write();
-
    for (Int_t i = 0; i < numLayers-1; i++) {
       
       TObjArray* layer1 = (TObjArray*)fNetwork->At(i);
@@ -616,7 +920,7 @@ void TMVA::MethodANNBase::WriteMonitoringHistosToFile() const
       Int_t numNeurons1 = layer1->GetEntriesFast();
       Int_t numNeurons2 = layer2->GetEntriesFast();
       
-      TString name = Form("weights_hist%i%i", i, i+1);
+      TString name = Form("%s%i%i", bulkname.Data(), i, i+1);
       hist = new TH2F(name + "", name + "", 
                       numNeurons1, 0, numNeurons1, numNeurons2, 0, numNeurons2);
       
@@ -633,11 +937,51 @@ void TMVA::MethodANNBase::WriteMonitoringHistosToFile() const
          }
       }
       
-      hist->Write();
-      delete hist;
+      if (hv) hv->push_back( hist );
+      else {
+         hist->Write();
+         delete hist;
+      }
    }
 }
 
+//_______________________________________________________________________
+void TMVA::MethodANNBase::WriteMonitoringHistosToFile() const
+{
+   // write histograms to file
+   PrintMessage(Form("Write special histos to file: %s", BaseDir()->GetPath()), kTRUE);
+
+   if (fEstimatorHistTrain) fEstimatorHistTrain->Write();
+   if (fEstimatorHistTest ) fEstimatorHistTest ->Write();
+
+   // histograms containing weights for architecture plotting (used in macro "network.C")
+   CreateWeightMonitoringHists( "weights_hist" );
+
+   // now save all the epoch-wise monitoring information
+   static int epochMonitoringDirectoryNumber = 0;
+   TDirectory* epochdir = NULL;
+   if( epochMonitoringDirectoryNumber == 0 )
+      epochdir = BaseDir()->mkdir( "EpochMonitoring" );
+   else
+      epochdir = BaseDir()->mkdir( Form("EpochMonitoring_%4d",epochMonitoringDirectoryNumber) );
+   ++epochMonitoringDirectoryNumber;
+
+   epochdir->cd();
+   for (std::vector<TH1*>::const_iterator it = fEpochMonHistS.begin(); it != fEpochMonHistS.end(); it++) {
+      (*it)->Write();
+      delete (*it);
+   }
+   for (std::vector<TH1*>::const_iterator it = fEpochMonHistB.begin(); it != fEpochMonHistB.end(); it++) {
+      (*it)->Write();
+      delete (*it);
+   }
+   for (std::vector<TH1*>::const_iterator it = fEpochMonHistW.begin(); it != fEpochMonHistW.end(); it++) {
+      (*it)->Write();
+      delete (*it);
+   }
+   BaseDir()->cd();
+}
+
 //_______________________________________________________________________
 void TMVA::MethodANNBase::MakeClassSpecific( std::ostream& fout, const TString& className ) const
 {
@@ -646,6 +990,7 @@ void TMVA::MethodANNBase::MakeClassSpecific( std::ostream& fout, const TString&
 
    fout << endl;
    fout << "   double ActivationFnc(double x) const;" << endl;
+   fout << "   double OutputActivationFnc(double x) const;" << endl;     //zjh
    fout << endl;
    fout << "   int fLayers;" << endl;
    fout << "   int fLayerSize["<<numLayers<<"];" << endl;
@@ -728,8 +1073,9 @@ void TMVA::MethodANNBase::MakeClassSpecific( std::ostream& fout, const TString&
          fout << "         fWeights[" << i+1 << "][o] += fabs(inputVal);" << endl;
       }
       fout << "      }" << endl;
-      if (i+1 != numLayers-1) // in the last layer no activation funciton is applied
+      if (i+1 != numLayers-1) // in the last layer no activation function is applied
          fout << "      fWeights[" << i+1 << "][o] = ActivationFnc(fWeights[" << i+1 << "][o]);" << endl;
+      else	fout << "      fWeights[" << i+1 << "][o] = OutputActivationFnc(fWeights[" << i+1 << "][o]);" << endl; //zjh
       fout << "   }" << endl;
    }
    fout << endl;
@@ -739,6 +1085,8 @@ void TMVA::MethodANNBase::MakeClassSpecific( std::ostream& fout, const TString&
    fout << endl;
    TString fncName = className+"::ActivationFnc";
    fActivation->MakeFunction(fout, fncName);
+   fncName = className+"::OutputActivationFnc";  	//zjh
+   fOutput->MakeFunction(fout, fncName); 			//zjh
 
    fout << "   " << endl;
    fout << "// Clean up" << endl;
@@ -747,3 +1095,10 @@ void TMVA::MethodANNBase::MakeClassSpecific( std::ostream& fout, const TString&
    fout << "   // nothing to clear" << endl;
    fout << "}" << endl;
 }
+
+//_________________________________________________________________________
+Bool_t TMVA::MethodANNBase::Debug() const 
+{ 
+   // who the hell makes such strange Debug flags that even use "global pointers"..
+   return fgDEBUG; 
+}
