@@ -1,5 +1,5 @@
-// @(#)root/tmva $Id: MethodCFMlpANN_Utils.cxx 20882 2007-11-19 11:31:26Z rdm $ 
-// Author: Andreas Hoecker, Joerg Stelzer, Helge Voss, Kai Voss 
+// @(#)root/tmva $Id: MethodCFMlpANN_Utils.cxx 41891 2011-11-10 22:46:31Z pcanal $
+// Author: Andreas Hoecker, Joerg Stelzer, Helge Voss, Kai Voss
 
 /**********************************************************************************
  * Project: TMVA - a Root-integrated toolkit for multivariate data analysis       *
@@ -35,9 +35,9 @@
  *      Kai Voss        <Kai.Voss@cern.ch>       - U. of Victoria, Canada         *
  *                                                                                *
  * Copyright (c) 2005:                                                            *
- *      CERN, Switzerland                                                         * 
- *      U. of Victoria, Canada                                                    * 
- *      MPI-K Heidelberg, Germany                                                 * 
+ *      CERN, Switzerland                                                         *
+ *      U. of Victoria, Canada                                                    *
+ *      MPI-K Heidelberg, Germany                                                 *
  *      LAPP, Annecy, France                                                      *
  *                                                                                *
  * Redistribution and use in source and binary forms, with or without             *
@@ -47,33 +47,91 @@
  **********************************************************************************/
 
 //_______________________________________________________________________
-//                                                                      
-// Implementation of Clermond-Ferrand artificial neural network 
 //
-// Reference for the original FORTRAN version "mlpl3.F":              
+// Implementation of Clermond-Ferrand artificial neural network
+//
+// Reference for the original FORTRAN version "mlpl3.F":
 //      Authors  : J. Proriol and contributions from ALEPH-Clermont-Ferrand
-//                 Team members                               
-//      Copyright: Laboratoire Physique Corpusculaire         
-//                 Universite de Blaise Pascal, IN2P3/CNRS    
+//                 Team members
+//      Copyright: Laboratoire Physique Corpusculaire
+//                 Universite de Blaise Pascal, IN2P3/CNRS
 //_______________________________________________________________________
 
+#include <string>
 #include <iostream>
+#include <cstdlib>
+
+#include "TMath.h"
+#include "TString.h"
 
 #include "TMVA/MethodCFMlpANN_Utils.h"
 #include "TMVA/Timer.h"
-#include "TMath.h"
+
+using std::cout;
+using std::endl;
 
 ClassImp(TMVA::MethodCFMlpANN_Utils)
    
-Int_t TMVA::MethodCFMlpANN_Utils::fg_100         = 100;
-Int_t TMVA::MethodCFMlpANN_Utils::fg_0           = 0;
-Int_t TMVA::MethodCFMlpANN_Utils::fg_max_nVar_   = max_nVar_;
-Int_t TMVA::MethodCFMlpANN_Utils::fg_max_nNodes_ = max_nNodes_;
-Int_t TMVA::MethodCFMlpANN_Utils::fg_999         = 999;
+Int_t       TMVA::MethodCFMlpANN_Utils::fg_100         = 100;
+Int_t       TMVA::MethodCFMlpANN_Utils::fg_0           = 0;
+Int_t       TMVA::MethodCFMlpANN_Utils::fg_max_nVar_   = max_nVar_;
+Int_t       TMVA::MethodCFMlpANN_Utils::fg_max_nNodes_ = max_nNodes_;
+Int_t       TMVA::MethodCFMlpANN_Utils::fg_999         = 999;
+const char* TMVA::MethodCFMlpANN_Utils::fg_MethodName  = "--- CFMlpANN                 ";
 
-TMVA::MethodCFMlpANN_Utils::MethodCFMlpANN_Utils()  
+TMVA::MethodCFMlpANN_Utils::MethodCFMlpANN_Utils()
 {
    // default constructor
+   Int_t i(0);
+   for(i=0; i<max_nVar_;++i) fVarn_1.xmin[i] = 0;
+   fCost_1.ancout = 0;
+   fCost_1.ieps = 0;
+   fCost_1.tolcou = 0;
+
+   for(i=0; i<max_nNodes_;++i) fDel_1.coef[i] = 0;
+   for(i=0; i<max_nLayers_*max_nNodes_;++i) fDel_1.del[i] = 0;
+   for(i=0; i<max_nLayers_*max_nNodes_*max_nNodes_;++i) fDel_1.delta[i] = 0;
+   for(i=0; i<max_nLayers_*max_nNodes_*max_nNodes_;++i) fDel_1.delw[i] = 0;
+   for(i=0; i<max_nLayers_*max_nNodes_;++i) fDel_1.delww[i] = 0;
+   fDel_1.demin = 0;
+   fDel_1.demax = 0;
+   fDel_1.idde = 0;
+   for(i=0; i<max_nLayers_;++i) fDel_1.temp[i] = 0;
+
+   for(i=0; i<max_nNodes_;++i) fNeur_1.cut[i] = 0;
+   for(i=0; i<max_nLayers_*max_nNodes_;++i) fNeur_1.deltaww[i] = 0;
+   for(i=0; i<max_nLayers_;++i) fNeur_1.neuron[i] = 0;
+   for(i=0; i<max_nNodes_;++i) fNeur_1.o[i] = 0;
+   for(i=0; i<max_nLayers_*max_nNodes_*max_nNodes_;++i) fNeur_1.w[i] = 0;
+   for(i=0; i<max_nLayers_*max_nNodes_;++i) fNeur_1.ww[i] = 0;
+   for(i=0; i<max_nLayers_*max_nNodes_;++i) fNeur_1.x[i] = 0;
+   for(i=0; i<max_nLayers_*max_nNodes_;++i) fNeur_1.y[i] = 0;
+      
+   fParam_1.eeps = 0;
+   fParam_1.epsmin = 0;
+   fParam_1.epsmax = 0;
+   fParam_1.eta = 0;
+   fParam_1.ichoi = 0;
+   fParam_1.itest = 0;
+   fParam_1.layerm = 0;
+   fParam_1.lclass = 0;
+   fParam_1.nblearn = 0;
+   fParam_1.ndiv = 0;
+   fParam_1.ndivis = 0;
+   fParam_1.nevl = 0;
+   fParam_1.nevt = 0;
+   fParam_1.nunap = 0;
+   fParam_1.nunilec = 0;
+   fParam_1.nunishort = 0;
+   fParam_1.nunisor = 0;
+   fParam_1.nvar = 0;
+
+   fVarn_1.iclass = 0;
+   for(i=0; i<max_Events_;++i) fVarn_1.mclass[i] = 0;
+   for(i=0; i<max_Events_;++i) fVarn_1.nclass[i] = 0;
+   for(i=0; i<max_nVar_;++i) fVarn_1.xmax[i] = 0;
+
+   fLogger = 0;
 }
 
 TMVA::MethodCFMlpANN_Utils::~MethodCFMlpANN_Utils() 
@@ -97,7 +155,7 @@ void TMVA::MethodCFMlpANN_Utils::Train_nn( Double_t *tin2, Double_t *tout2, Int_
    if (*nvar2 > max_nVar_) {
       printf( "*** CFMlpANN_f2c: ERROR in Train_nn: number of variables" \
               " exceeds hardcoded maximum ==> abort");
-      exit(1);
+      std::exit(1);
    }
    if (*nlayer > max_nLayers_) {
       printf( "*** CFMlpANN_f2c: Warning in Train_nn: number of layers" \
@@ -114,16 +172,16 @@ void TMVA::MethodCFMlpANN_Utils::Train_nn( Double_t *tin2, Double_t *tout2, Int_
    fVarn2_1.Create( *ntrain + *ntest, *nvar2 );
    fVarn3_1.Create( *ntrain + *ntest, *nvar2 );
 
-   Int_t imax;
+   // Int_t imax;
    char det[20];
 
    Entree_new(nvar2, det, ntrain, ntest, nlayer, nodes, ncycle, (Int_t)20);
    if (fNeur_1.neuron[fParam_1.layerm - 1] == 1) {
-      imax = 2;
+      // imax = 2;
       fParam_1.lclass = 2;
    } 
    else {
-      imax = fNeur_1.neuron[fParam_1.layerm - 1] << 1;
+      // imax = fNeur_1.neuron[fParam_1.layerm - 1] << 1;
       fParam_1.lclass = fNeur_1.neuron[fParam_1.layerm - 1];
    }
    fParam_1.nvar = fNeur_1.neuron[0];
@@ -163,7 +221,7 @@ void TMVA::MethodCFMlpANN_Utils::Entree_new( Int_t *, char *, Int_t *ntrain,
    }
    fParam_1.layerm = *numlayer;
    if (fParam_1.layerm > max_nLayers_) {
-      printf("Error: number of layers exceeds maximum: %i, %i ==> abort", 
+      printf("Error: number of layers exceeds maximum: %i, %i ==> abort",
              fParam_1.layerm, max_nLayers_ );
       Arret("modification of mlpl3_param_lim.inc is needed ");
    }
@@ -180,16 +238,16 @@ void TMVA::MethodCFMlpANN_Utils::Entree_new( Int_t *, char *, Int_t *ntrain,
    fParam_1.nunisor = 30;
    fParam_1.nunishort = 48;
    fParam_1.nunap = 40;
-   printf("--- CFMlpANN     : Total number of events for training: %i\n", fParam_1.nevl);
-   printf("--- CFMlpANN     : Total number of events for testing : %i\n", fParam_1.nevt);
-   printf("--- CFMlpANN     : Total number of training cycles    : %i\n", fParam_1.nblearn);
+   
+   ULog() << kINFO << "Total number of events for training: " << fParam_1.nevl << Endl;
+   ULog() << kINFO << "Total number of training cycles    : " << fParam_1.nblearn << Endl;
    if (fParam_1.nevl > max_Events_) {
-      printf("Error: number of learning events exceeds maximum: %i, %i ==> abort", 
+      printf("Error: number of learning events exceeds maximum: %i, %i ==> abort",
              fParam_1.nevl, max_Events_ );
       Arret("modification of mlpl3_param_lim.inc is needed ");
    }
    if (fParam_1.nevt > max_Events_) {
-      printf("Error: number of testing events exceeds maximum: %i, %i ==> abort", 
+      printf("Error: number of testing events exceeds maximum: %i, %i ==> abort",
              fParam_1.nevt, max_Events_ );
       Arret("modification of mlpl3_param_lim.inc is needed ");
    }
@@ -206,8 +264,7 @@ void TMVA::MethodCFMlpANN_Utils::Entree_new( Int_t *, char *, Int_t *ntrain,
    }
    i__1 = fParam_1.layerm;
    for (j = 1; j <= i__1; ++j) {
-      printf("--- CFMlpANN     : Number of layers for neuron(%2i): %i\n",j, 
-             fNeur_1.neuron[j - 1]);
+      ULog() << kINFO << "Number of layers for neuron(" << j << "): " << fNeur_1.neuron[j - 1] << Endl;
    }
    if (fNeur_1.neuron[fParam_1.layerm - 1] != 2) {
       printf("Error: wrong number of classes at ouput layer: %i != 2 ==> abort\n",
@@ -230,10 +287,10 @@ void TMVA::MethodCFMlpANN_Utils::Entree_new( Int_t *, char *, Int_t *ntrain,
       Arret("new training or continued one !");
    }
    if (fParam_1.ichoi == 0) {
-      printf("--- CFMlpANN     : New training will be performed\n");
-   } 
+      ULog() << kINFO << "New training will be performed" << Endl;
+   }
    else {
-      printf("--- CFMlpANN     : New training will be continued from a weight file\n");
+      printf("%s: New training will be continued from a weight file\n", fg_MethodName);
    }
    ncoef = 0;
    ntemp = 0;
@@ -523,7 +580,7 @@ void TMVA::MethodCFMlpANN_Utils::Innit( char *det, Double_t *tout2, Double_t *ti
 
    Int_t i__, j;
    Int_t nevod, layer, ktest, i1, nrest;
-   Int_t ievent;
+   Int_t ievent(0);
    Int_t kkk;
    Double_t xxx, yyy;
 
@@ -531,7 +588,7 @@ void TMVA::MethodCFMlpANN_Utils::Innit( char *det, Double_t *tout2, Double_t *ti
    Lecev2(&ktest, tout2, tin2);
    if (ktest == 1) {
       printf( " .... strange to be here (1) ... \n");
-      exit(1);
+      std::exit(1);
    }
    i__1 = fParam_1.layerm - 1;
    for (layer = 1; layer <= i__1; ++layer) {
@@ -557,7 +614,7 @@ void TMVA::MethodCFMlpANN_Utils::Innit( char *det, Double_t *tout2, Double_t *ti
 
    for (i1 = 1; i1 <= i__3; ++i1) {
 
-      if ( num>0 && (i1-1)%num == 0 || i1 == i__3) timer.DrawProgressBar( i1-1 );
+      if ( ( num>0 && (i1-1)%num == 0) || (i1 == i__3) ) timer.DrawProgressBar( i1-1 );
 
       i__2 = fParam_1.nevl;
       for (i__ = 1; i__ <= i__2; ++i__) {
@@ -605,8 +662,6 @@ void TMVA::MethodCFMlpANN_Utils::Innit( char *det, Double_t *tout2, Double_t *ti
          break;
       }
    }
-   printf("--- CFMlpANN     : Elapsed time: %s\n", (const char*)timer.GetElapsedTime()  );
-
 }
 
 #undef deltaww_ref
@@ -660,7 +715,7 @@ void TMVA::MethodCFMlpANN_Utils::TestNN()
    }
    if (ktest == 1) {
       printf( " .... strange to be here (2) ... \n");
-      exit(1);
+      std::exit(1);
    }
 }
 
@@ -705,9 +760,9 @@ void TMVA::MethodCFMlpANN_Utils::Cout( Int_t * /*i1*/, Double_t *xxx )
 void TMVA::MethodCFMlpANN_Utils::Inl()
 {
    // [smart comments to be added]
-   Int_t i__1, i__2, i__3;
+   Int_t i__1, i__2;
 
-   Int_t jmin, jmax, k, layer, kk, nq, nr;
+   Int_t jmax, k, layer, kk, nq, nr;
 
    i__1 = fParam_1.nvar;
    i__1 = fParam_1.layerm;
@@ -723,12 +778,12 @@ void TMVA::MethodCFMlpANN_Utils::Inl()
       }
       i__2 = kk;
       for (k = 1; k <= i__2; ++k) {
-         jmin = k * 10 - 9;
+         // jmin = k * 10 - 9;
          jmax = k * 10;
          if (fNeur_1.neuron[layer] < jmax) {
             jmax = fNeur_1.neuron[layer];
          }
-         i__3 = fNeur_1.neuron[layer - 1];
+         // i__3 = fNeur_1.neuron[layer - 1];
       }
    }
 }
@@ -759,17 +814,17 @@ void TMVA::MethodCFMlpANN_Utils::GraphNN( Int_t *ilearn, Double_t * /*xxx*/,
    Int_t i__1, i__2;
    
    Double_t xmok[max_nNodes_];
-   Float_t xpaw;
+   // Float_t xpaw;
    Double_t xmko[max_nNodes_];
    Int_t i__, j;
    Int_t ix;
-   Int_t jjj;
-   Float_t vbn[10];
+   // Int_t jjj;
+   // Float_t vbn[10];
    Int_t nko[max_nNodes_], nok[max_nNodes_];
 
-   for (i__ = 1; i__ <= 10; ++i__) {
-      vbn[i__ - 1] = (Float_t)0.;
-   }
+   // for (i__ = 1; i__ <= 10; ++i__) {
+   //    vbn[i__ - 1] = (Float_t)0.;
+   // }
    if (*ilearn == 1) {
       // AH: removed output 
    }
@@ -785,7 +840,7 @@ void TMVA::MethodCFMlpANN_Utils::GraphNN( Int_t *ilearn, Double_t * /*xxx*/,
       En_avant(&i__);
       i__2 = fNeur_1.neuron[fParam_1.layerm - 1];
       for (j = 1; j <= i__2; ++j) {
-         xpaw = (Float_t) y_ref(fParam_1.layerm, j);
+         // xpaw = (Float_t) y_ref(fParam_1.layerm, j);
          if (fVarn_1.nclass[i__ - 1] == j) {
             ++nok[j - 1];
             xmok[j - 1] += y_ref(fParam_1.layerm, j);
@@ -793,13 +848,13 @@ void TMVA::MethodCFMlpANN_Utils::GraphNN( Int_t *ilearn, Double_t * /*xxx*/,
          else {
             ++nko[j - 1];
             xmko[j - 1] += y_ref(fParam_1.layerm, j);
-            jjj = j + fNeur_1.neuron[fParam_1.layerm - 1];
-         }
-         if (j <= 9) {
-            vbn[j - 1] = xpaw;
+            // jjj = j + fNeur_1.neuron[fParam_1.layerm - 1];
          }
+         // if (j <= 9) {
+         //    vbn[j - 1] = xpaw;
+         // }
       }
-      vbn[9] = (Float_t) fVarn_1.nclass[i__ - 1];
+      // vbn[9] = (Float_t) fVarn_1.nclass[i__ - 1];
    }
    i__1 = fNeur_1.neuron[fParam_1.layerm - 1];
    for (j = 1; j <= i__1; ++j) {
@@ -907,7 +962,9 @@ void TMVA::MethodCFMlpANN_Utils::Lecev2( Int_t *ktest, Double_t *tout2, Double_t
    // [smart comments to be added]
    Int_t i__1, i__2;
 
-   Int_t i__, j, k, l, mocla[max_nNodes_], ikend;
+   Int_t i__, j, l;
+   // Int_t  mocla[max_nNodes_];
+   Int_t ikend;
    Double_t xpg[max_nVar_];
 
    /* NTRAIN: Nb of events used during the learning */
@@ -917,9 +974,9 @@ void TMVA::MethodCFMlpANN_Utils::Lecev2( Int_t *ktest, Double_t *tout2, Double_t
 
    *ktest = 0;
    i__1 = fParam_1.lclass;
-   for (k = 1; k <= i__1; ++k) {
-      mocla[k - 1] = 0;
-   }
+   // for (k = 1; k <= i__1; ++k) {
+   //    mocla[k - 1] = 0;
+   // }
    i__1 = fParam_1.nevt;
    for (i__ = 1; i__ <= i__1; ++i__) {
       DataInterface(tout2, tin2, &fg_999, &fg_0, &fParam_1.nevt, &fParam_1.nvar, 
@@ -1002,27 +1059,27 @@ void TMVA::MethodCFMlpANN_Utils::En_avant2( Int_t *ievent )
 void TMVA::MethodCFMlpANN_Utils::Arret( const char* mot )
 {
    // fatal error occurred: stop execution
-   printf("CFMlpANN: %s",mot);
-   exit(1);
+   printf("%s: %s",fg_MethodName, mot);
+   std::exit(1);
 }
 
-void TMVA::MethodCFMlpANN_Utils::CollectVar( Int_t *nvar, Int_t *class__, Double_t *xpg )
+void TMVA::MethodCFMlpANN_Utils::CollectVar( Int_t * /*nvar*/, Int_t * /*class__*/, Double_t * /*xpg*/ )
 {
-   // [smart comments to be added]
-   Int_t i__1;
+   // // [smart comments to be added]
+   // Int_t i__1;
    
-   Int_t i__;
-   Float_t x[201];
+   // Int_t i__;
+   // Float_t x[201];
 
-   // Parameter adjustments
-   --xpg;
+   // // Parameter adjustments
+   // --xpg;
 
-   for (i__ = 1; i__ <= 201; ++i__) {
-      x[i__ - 1] = 0.0;
-   }
-   x[0] = (Float_t) (*class__);
-   i__1 = *nvar;
-   for (i__ = 1; i__ <= i__1; ++i__) {
-      x[i__] = (Float_t) xpg[i__];
-   }
+   // for (i__ = 1; i__ <= 201; ++i__) {
+   //    x[i__ - 1] = 0.0;
+   // }
+   // x[0] = (Float_t) (*class__);
+   // i__1 = *nvar;
+   // for (i__ = 1; i__ <= i__1; ++i__) {
+   //    x[i__] = (Float_t) xpg[i__];
+   // }
 }
