@@ -1,5 +1,5 @@
-// @(#)root/tmva $Id: MethodHMatrix.cxx 21630 2008-01-10 19:40:44Z brun $    
-// Author: Andreas Hoecker, Joerg Stelzer, Helge Voss, Kai Voss 
+// @(#)root/tmva $Id: MethodHMatrix.cxx 38475 2011-03-17 10:46:00Z evt $
+// Author: Andreas Hoecker, Joerg Stelzer, Helge Voss, Kai Voss
 
 /**********************************************************************************
  * Project: TMVA - a Root-integrated toolkit for multivariate Data analysis       *
@@ -12,87 +12,84 @@
  *                                                                                *
  * Authors (alphabetical):                                                        *
  *      Andreas Hoecker <Andreas.Hocker@cern.ch> - CERN, Switzerland              *
- *      Xavier Prudent  <prudent@lapp.in2p3.fr>  - LAPP, France                   *
+ *      Peter Speckmayer <Peter.Speckmayer@cern.ch> - CERN, Switzerland           *
  *      Helge Voss      <Helge.Voss@cern.ch>     - MPI-K Heidelberg, Germany      *
  *      Kai Voss        <Kai.Voss@cern.ch>       - U. of Victoria, Canada         *
  *                                                                                *
  * Copyright (c) 2005:                                                            *
- *      CERN, Switzerland                                                         * 
- *      U. of Victoria, Canada                                                    * 
- *      MPI-K Heidelberg, Germany                                                 * 
- *      LAPP, Annecy, France                                                      *
+ *      CERN, Switzerland                                                         *
+ *      U. of Victoria, Canada                                                    *
+ *      MPI-K Heidelberg, Germany                                                 *
  *                                                                                *
  * Redistribution and use in source and binary forms, with or without             *
  * modification, are permitted according to the terms listed in LICENSE           *
  * (http://tmva.sourceforge.net/LICENSE)                                          *
  **********************************************************************************/
 
+#include "TMVA/ClassifierFactory.h"
 #include "TMVA/MethodHMatrix.h"
 #include "TMVA/Tools.h"
 #include "TMatrix.h"
 #include "Riostream.h"
 #include <algorithm>
 
+REGISTER_METHOD(HMatrix)
+
 ClassImp(TMVA::MethodHMatrix)
 
 //_______________________________________________________________________
-/* Begin_Html                                                                      
-  H-Matrix method, which is implemented as a simple comparison of      
-  chi-squared estimators for signal and background, taking into        
-  account the linear correlations between the input variables          
+//Begin_Html
+/*
+  H-Matrix method, which is implemented as a simple comparison of
+  chi-squared estimators for signal and background, taking into
+  account the linear correlations between the input variables
 
-  This MVA approach is used by the D&#216; collaboration (FNAL) for the 
-  purpose of electron identification (see, eg., 
-  <a href="http://arxiv.org/abs/hep-ex/9507007">hep-ex/9507007</a>). 
+  This MVA approach is used by the D&#216; collaboration (FNAL) for the
+  purpose of electron identification (see, eg.,
+  <a href="http://arxiv.org/abs/hep-ex/9507007">hep-ex/9507007</a>).
   As it is implemented in TMVA, it is usually equivalent or worse than
-  the Fisher-Mahalanobis discriminant, and it has only been added for 
+  the Fisher-Mahalanobis discriminant, and it has only been added for
   the purpose of completeness.
   Two &chi;<sup>2</sup> estimators are computed for an event, each one
-  for signal and background, using the estimates for the means and 
+  for signal and background, using the estimates for the means and
   covariance matrices obtained from the training sample:<br>
   <center>
-  <img vspace=6 src="gif/tmva_chi2.gif" align="bottom" > 
+  <img vspace=6 src="gif/tmva_chi2.gif" align="bottom" >
   </center>
   TMVA then uses as normalised analyser for event (<i>i</i>) the ratio:
   (<i>&chi;<sub>S</sub>(i)<sup>2</sup> &minus; &chi;<sub>B</sub><sup>2</sup>(i)</i>)
   (<i>&chi;<sub>S</sub><sup>2</sup>(i) + &chi;<sub>B</sub><sup>2</sup>(i)</i>).
-
-End_Html */
+*/
+//End_Html
 //_______________________________________________________________________
- 
+
 
 //_______________________________________________________________________
-TMVA::MethodHMatrix::MethodHMatrix( const TString& jobName, const TString& methodTitle, DataSet& theData, 
-                                    const TString& theOption, TDirectory* theTargetDir )
-   : TMVA::MethodBase( jobName, methodTitle, theData, theOption, theTargetDir )
+TMVA::MethodHMatrix::MethodHMatrix( const TString& jobName,
+                                    const TString& methodTitle,
+                                    DataSetInfo& theData,
+                                    const TString& theOption,
+                                    TDirectory* theTargetDir )
+   : TMVA::MethodBase( jobName, Types::kHMatrix, methodTitle, theData, theOption, theTargetDir )
 {
    // standard constructor for the H-Matrix method
-   InitHMatrix();
-
-   // HMatrix options: none ... process base class option
-   ProcessOptions();
 }
 
 //_______________________________________________________________________
-TMVA::MethodHMatrix::MethodHMatrix( DataSet & theData, 
-                                    const TString& theWeightFile,  
+TMVA::MethodHMatrix::MethodHMatrix( DataSetInfo& theData,
+                                    const TString& theWeightFile,
                                     TDirectory* theTargetDir )
-   : TMVA::MethodBase( theData, theWeightFile, theTargetDir ) 
+   : TMVA::MethodBase( Types::kHMatrix, theData, theWeightFile, theTargetDir )
 {
-   // constructor to calculate the H-Matrix from the weight file
-   InitHMatrix();
+   // constructor from weight file
 }
 
 //_______________________________________________________________________
-void TMVA::MethodHMatrix::InitHMatrix( void )
+void TMVA::MethodHMatrix::Init( void )
 {
    // default initialization called by all constructors
-   SetMethodName( "HMatrix" );
-   SetMethodType( TMVA::Types::kHMatrix );
-   SetTestvarName();
 
-   // HMatrix prefers normalised input variables
-   SetNormalised( kTRUE );
+   //SetNormalised( kFALSE ); obsolete!
 
    fInvHMatrixS = new TMatrixD( GetNvar(), GetNvar() );
    fInvHMatrixB = new TMatrixD( GetNvar(), GetNvar() );
@@ -114,16 +111,24 @@ TMVA::MethodHMatrix::~MethodHMatrix( void )
 }
 
 //_______________________________________________________________________
-void TMVA::MethodHMatrix::DeclareOptions() 
+Bool_t TMVA::MethodHMatrix::HasAnalysisType( Types::EAnalysisType type, UInt_t numberClasses, UInt_t /*numberTargets*/ )
+{
+   // FDA can handle classification with 2 classes and regression with one regression-target
+   if( type == Types::kClassification && numberClasses == 2 ) return kTRUE;
+   return kFALSE;
+}
+
+
+//_______________________________________________________________________
+void TMVA::MethodHMatrix::DeclareOptions()
 {
    // MethodHMatrix options: none (apart from those implemented in MethodBase)
 }
 
 //_______________________________________________________________________
-void TMVA::MethodHMatrix::ProcessOptions() 
+void TMVA::MethodHMatrix::ProcessOptions()
 {
    // process user options
-   MethodBase::ProcessOptions();
 }
 
 //_______________________________________________________________________
@@ -131,38 +136,35 @@ void TMVA::MethodHMatrix::Train( void )
 {
    // computes H-matrices for signal and background samples
 
-   // default sanity checks
-   if (!CheckSanity()) fLogger << kFATAL << "<Train> sanity check failed" << Endl;
-
    // covariance matrices for signal and background
    ComputeCovariance( kTRUE,  fInvHMatrixS );
    ComputeCovariance( kFALSE, fInvHMatrixB );
 
    // sanity checks
    if (TMath::Abs(fInvHMatrixS->Determinant()) < 10E-24) {
-      fLogger << kWARNING << "<Train> H-matrix  S is almost singular with deterinant= "
-              << TMath::Abs(fInvHMatrixS->Determinant())
-              << " did you use the variables that are linear combinations or highly correlated ???" 
-              << Endl;
+      Log() << kWARNING << "<Train> H-matrix  S is almost singular with deterinant= "
+            << TMath::Abs(fInvHMatrixS->Determinant())
+            << " did you use the variables that are linear combinations or highly correlated ???"
+            << Endl;
    }
    if (TMath::Abs(fInvHMatrixB->Determinant()) < 10E-24) {
-      fLogger << kWARNING << "<Train> H-matrix  B is almost singular with deterinant= "
-              << TMath::Abs(fInvHMatrixB->Determinant())
-              << " did you use the variables that are linear combinations or highly correlated ???" 
-              << Endl;
+      Log() << kWARNING << "<Train> H-matrix  B is almost singular with deterinant= "
+            << TMath::Abs(fInvHMatrixB->Determinant())
+            << " did you use the variables that are linear combinations or highly correlated ???"
+            << Endl;
    }
 
     if (TMath::Abs(fInvHMatrixS->Determinant()) < 10E-120) {
-       fLogger << kFATAL << "<Train> H-matrix  S is singular with deterinant= "
-               << TMath::Abs(fInvHMatrixS->Determinant())
-               << " did you use the variables that are linear combinations ???" 
-               << Endl;
+       Log() << kFATAL << "<Train> H-matrix  S is singular with deterinant= "
+             << TMath::Abs(fInvHMatrixS->Determinant())
+             << " did you use the variables that are linear combinations ???"
+             << Endl;
     }
     if (TMath::Abs(fInvHMatrixB->Determinant()) < 10E-120) {
-       fLogger << kFATAL << "<Train> H-matrix  B is singular with deterinant= "
-               << TMath::Abs(fInvHMatrixB->Determinant())
-               << " did you use the variables that are linear combinations ???" 
-               << Endl;
+       Log() << kFATAL << "<Train> H-matrix  B is singular with deterinant= "
+             << TMath::Abs(fInvHMatrixB->Determinant())
+             << " did you use the variables that are linear combinations ???"
+             << Endl;
     }
 
    // invert matrix
@@ -175,7 +177,9 @@ void TMVA::MethodHMatrix::ComputeCovariance( Bool_t isSignal, TMatrixD* mat )
 {
    // compute covariance matrix
 
-   const UInt_t nvar = GetNvar();
+   Data()->SetCurrentType(Types::kTraining);
+
+   const UInt_t nvar = DataInfo().GetNVariables();
    UInt_t ivar, jvar;
 
    // init matrices
@@ -187,30 +191,38 @@ void TMVA::MethodHMatrix::ComputeCovariance( Bool_t isSignal, TMatrixD* mat )
    Double_t *xval = new Double_t[nvar];
 
    // perform event loop
-   for (Int_t i=0; i<Data().GetNEvtTrain(); i++) {
+   for (Int_t i=0, iEnd=Data()->GetNEvents(); i<iEnd; ++i) {
 
-      // fill the event
-      ReadTrainingEvent(i);      
-      if (IsSignalEvent() != isSignal) continue;
+      // retrieve the original (not transformed) event
+      const Event* origEvt = Data()->GetEvent(i);
+      Double_t weight = origEvt->GetWeight();
 
+      // in case event with neg weights are to be ignored
+      if (IgnoreEventsWithNegWeightsInTraining() && weight <= 0) continue;
+
+      if (DataInfo().IsSignal(origEvt) != isSignal) continue;
+
+      // transform the event
+      GetTransformationHandler().SetTransformationReferenceClass( origEvt->GetClass() );
+      const Event* ev = GetTransformationHandler().Transform( origEvt );
+      
       // event is of good type
-      Double_t weight = GetEventWeight();
       sumOfWeights += weight;
 
       // mean values
-      for (ivar=0; ivar<nvar; ivar++) xval[ivar] = GetEventVal(ivar);
+      for (ivar=0; ivar<nvar; ivar++) xval[ivar] = ev->GetValue(ivar);
 
-      // covariance matrix         
+      // covariance matrix
       for (ivar=0; ivar<nvar; ivar++) {
 
          vec(ivar)        += xval[ivar]*weight;
          mat2(ivar, ivar) += (xval[ivar]*xval[ivar])*weight;
-         
+
          for (jvar=ivar+1; jvar<nvar; jvar++) {
             mat2(ivar, jvar) += (xval[ivar]*xval[jvar])*weight;
             mat2(jvar, ivar) = mat2(ivar, jvar); // symmetric matrix
          }
-      }         
+      }
    }
 
    // variance-covariance
@@ -228,33 +240,46 @@ void TMVA::MethodHMatrix::ComputeCovariance( Bool_t isSignal, TMatrixD* mat )
 }
 
 //_______________________________________________________________________
-Double_t TMVA::MethodHMatrix::GetMvaValue()
+Double_t TMVA::MethodHMatrix::GetMvaValue( Double_t* err, Double_t* errUpper )
 {
    // returns the H-matrix signal estimator
    Double_t s = GetChi2( Types::kSignal     );
    Double_t b = GetChi2( Types::kBackground );
   
-   if (s+b < 0) fLogger << kFATAL << "big trouble: s+b: " << s+b << Endl;
+   if (s+b < 0) Log() << kFATAL << "big trouble: s+b: " << s+b << Endl;
+
+   // cannot determine error
+   NoErrorCalc(err, errUpper);
 
    return (b - s)/(s + b);
 }
 
 //_______________________________________________________________________
-Double_t TMVA::MethodHMatrix::GetChi2( TMVA::Event* e,  Types::ESBType type ) const
+Double_t TMVA::MethodHMatrix::GetChi2( Types::ESBType type ) 
 {
    // compute chi2-estimator for event according to type (signal/background)
 
+   // get original (not transformed) event
+
+   const Event* origEvt = fTmpEvent ? fTmpEvent:Data()->GetEvent();
+
    // loop over variables
-   Int_t ivar,jvar;
-   vector<Double_t> val( GetNvar() );
-   for (ivar=0; ivar<GetNvar(); ivar++) {
-      val[ivar] = e->GetVal(ivar);
-      if (IsNormalised()) val[ivar] = Tools::NormVariable( val[ivar], GetXmin( ivar ), GetXmax( ivar ) );    
-   }
+   UInt_t ivar(0), jvar(0), nvar(GetNvar());
+   vector<Double_t> val( nvar );
+
+   // transform the event according to the given type (signal/background)
+   if (type==Types::kSignal)
+      GetTransformationHandler().SetTransformationReferenceClass( fSignalClass     );
+   else
+      GetTransformationHandler().SetTransformationReferenceClass( fBackgroundClass );
+
+   const Event* ev = GetTransformationHandler().Transform( origEvt );
+
+   for (ivar=0; ivar<nvar; ivar++) val[ivar] = ev->GetValue( ivar );
 
    Double_t chi2 = 0;
-   for (ivar=0; ivar<GetNvar(); ivar++) {
-      for (jvar=0; jvar<GetNvar(); jvar++) {
+   for (ivar=0; ivar<nvar; ivar++) {
+      for (jvar=0; jvar<nvar; jvar++) {
          if (type == Types::kSignal) 
             chi2 += ( (val[ivar] - (*fVecMeanS)(ivar))*(val[jvar] - (*fVecMeanS)(jvar))
                       * (*fInvHMatrixS)(ivar,jvar) );
@@ -265,68 +290,36 @@ Double_t TMVA::MethodHMatrix::GetChi2( TMVA::Event* e,  Types::ESBType type ) co
    }
 
    // sanity check
-   if (chi2 < 0) fLogger << kFATAL << "<GetChi2> negative chi2: " << chi2 << Endl;
+   if (chi2 < 0) Log() << kFATAL << "<GetChi2> negative chi2: " << chi2 << Endl;
 
    return chi2;
 }
 
 //_______________________________________________________________________
-Double_t TMVA::MethodHMatrix::GetChi2( Types::ESBType type ) const
+void TMVA::MethodHMatrix::AddWeightsXMLTo( void* parent ) const 
 {
-   // compute chi2-estimator for event according to type (signal/background)
+   // create XML description for HMatrix classification
 
-   // loop over variables
-   Int_t ivar,jvar;
-   vector<Double_t> val( GetNvar() );
-   for (ivar=0; ivar<GetNvar(); ivar++) val[ivar] = GetEventVal( ivar );
-
-   Double_t chi2 = 0;
-   for (ivar=0; ivar<GetNvar(); ivar++) {
-      for (jvar=0; jvar<GetNvar(); jvar++) {
-         if (type == Types::kSignal) 
-            chi2 += ( (val[ivar] - (*fVecMeanS)(ivar))*(val[jvar] - (*fVecMeanS)(jvar))
-                      * (*fInvHMatrixS)(ivar,jvar) );
-         else
-            chi2 += ( (val[ivar] - (*fVecMeanB)(ivar))*(val[jvar] - (*fVecMeanB)(jvar))
-                      * (*fInvHMatrixB)(ivar,jvar) );
-      }
-   }
-
-   // sanity check
-   if (chi2 < 0) fLogger << kFATAL << "<GetChi2> negative chi2: " << chi2 << Endl;
-
-   return chi2;
+   void* wght = gTools().AddChild(parent, "Weights");
+   gTools().WriteTVectorDToXML( wght, "VecMeanS", fVecMeanS    ); 
+   gTools().WriteTVectorDToXML( wght, "VecMeanB", fVecMeanB    );
+   gTools().WriteTMatrixDToXML( wght, "InvHMatS", fInvHMatrixS ); 
+   gTools().WriteTMatrixDToXML( wght, "InvHMatB", fInvHMatrixB );
 }
-  
+
 //_______________________________________________________________________
-void  TMVA::MethodHMatrix::WriteWeightsToStream( ostream& o ) const
-{  
-   // write variable names and min/max 
-   // NOTE: the latter values are mandatory for the normalisation 
-   // in the reader application !!!
-   Int_t ivar,jvar;
-   o << this->GetMethodName() <<endl;
+void TMVA::MethodHMatrix::ReadWeightsFromXML( void* wghtnode )
+{
+   // read weights from XML file
 
-   // mean vectors
-   for (ivar=0; ivar<GetNvar(); ivar++) {
-      o << (*fVecMeanS)(ivar) << "  " << (*fVecMeanB)(ivar) << endl;
-   }
-
-   // inverse covariance matrices (signal)
-   for (ivar=0; ivar<GetNvar(); ivar++) {
-      for (jvar=0; jvar<GetNvar(); jvar++) {
-         o << (*fInvHMatrixS)(ivar,jvar) << "  ";
-      }
-      o << endl;
-   }
-
-   // inverse covariance matrices (background)
-   for (ivar=0; ivar<GetNvar(); ivar++) {
-      for (jvar=0; jvar<GetNvar(); jvar++) {
-         o << (*fInvHMatrixB)(ivar,jvar) << "  ";
-      }
-      o << endl;
-   }
+   void* descnode = gTools().GetChild(wghtnode);
+   gTools().ReadTVectorDFromXML( descnode, "VecMeanS", fVecMeanS    );
+   descnode = gTools().GetNextChild(descnode);
+   gTools().ReadTVectorDFromXML( descnode, "VecMeanB", fVecMeanB    );
+   descnode = gTools().GetNextChild(descnode);
+   gTools().ReadTMatrixDFromXML( descnode, "InvHMatS", fInvHMatrixS ); 
+   descnode = gTools().GetNextChild(descnode);
+   gTools().ReadTMatrixDFromXML( descnode, "InvHMatB", fInvHMatrixB );
 }
 
 //_______________________________________________________________________
@@ -335,10 +328,10 @@ void  TMVA::MethodHMatrix::ReadWeightsFromStream( istream& istr )
    // read variable names and min/max
    // NOTE: the latter values are mandatory for the normalisation 
    // in the reader application !!!
-   Int_t ivar,jvar;
+   UInt_t ivar,jvar;
    TString var, dummy;
    istr >> dummy;
-   this->SetMethodName(dummy);
+   //this->SetMethodName(dummy);
 
    // mean vectors
    for (ivar=0; ivar<GetNvar(); ivar++) 
@@ -371,14 +364,14 @@ void TMVA::MethodHMatrix::MakeClassSpecific( std::ostream& fout, const TString&
    fout << "void " << className << "::Initialize() " << endl;
    fout << "{" << endl;
    fout << "   // init vectors with mean values" << endl;
-   for (Int_t ivar=0; ivar<GetNvar(); ivar++) {
+   for (UInt_t ivar=0; ivar<GetNvar(); ivar++) {
       fout << "   fVecMeanS[" << ivar << "] = " << (*fVecMeanS)(ivar) << ";" << endl;
       fout << "   fVecMeanB[" << ivar << "] = " << (*fVecMeanB)(ivar) << ";" << endl;
    }
    fout << "   " << endl;
    fout << "   // init H-matrices" << endl;
-   for (Int_t ivar=0; ivar<GetNvar(); ivar++) {
-      for (Int_t jvar=0; jvar<GetNvar(); jvar++) {
+   for (UInt_t ivar=0; ivar<GetNvar(); ivar++) {
+      for (UInt_t jvar=0; jvar<GetNvar(); jvar++) {
          fout << "   fInvHMatrixS[" << ivar << "][" << jvar << "] = " 
               << (*fInvHMatrixS)(ivar,jvar) << ";" << endl;
          fout << "   fInvHMatrixB[" << ivar << "][" << jvar << "] = " 
@@ -390,8 +383,24 @@ void TMVA::MethodHMatrix::MakeClassSpecific( std::ostream& fout, const TString&
    fout << "inline double " << className << "::GetMvaValue__( const std::vector<double>& inputValues ) const" << endl;
    fout << "{" << endl;
    fout << "   // returns the H-matrix signal estimator" << endl;
-   fout << "   double s = GetChi2( inputValues, " << Types::kSignal << " );" << endl;
-   fout << "   double b = GetChi2( inputValues, " << Types::kBackground << " );" << endl;
+   fout << "   std::vector<double> inputValuesSig = inputValues;" << endl;
+   fout << "   std::vector<double> inputValuesBgd = inputValues;" << endl;
+   if (GetTransformationHandler().GetTransformationList().GetSize() != 0) {
+
+      UInt_t signalClass    =DataInfo().GetClassInfo("Signal")->GetNumber();
+      UInt_t backgroundClass=DataInfo().GetClassInfo("Background")->GetNumber();
+
+      fout << "   Transform(inputValuesSig," << signalClass << ");" << endl;
+      fout << "   Transform(inputValuesBgd," << backgroundClass << ");" << endl;
+   }
+
+//   fout << "   for(uint i=0; i<GetNvar(); ++i) std::cout << inputValuesSig.at(i) << \"  \" << inputValuesBgd.at(i) << std::endl; " << endl;
+
+   fout << "   double s = GetChi2( inputValuesSig, " << Types::kSignal << " );" << endl;
+   fout << "   double b = GetChi2( inputValuesBgd, " << Types::kBackground << " );" << endl;
+
+//   fout << "   std::cout << s << \"  \" << b << std::endl; " << endl;
+
    fout << "   " << endl;
    fout << "   if (s+b <= 0) std::cout << \"Problem in class " << className << "::GetMvaValue__: s+b = \"" << endl;
    fout << "                           << s+b << \" <= 0 \"  << std::endl;" << endl;
@@ -437,27 +446,27 @@ void TMVA::MethodHMatrix::GetHelpMessage() const
    //
    // typical length of text line: 
    //         "|--------------------------------------------------------------|"
-   fLogger << Endl;
-   fLogger << Tools::Color("bold") << "--- Short description:" << Tools::Color("reset") << Endl;
-   fLogger << Endl;
-   fLogger << "The H-Matrix classifier discriminates one class (signal) of a feature" << Endl;
-   fLogger << "vector from another (background). The correlated elements of the" << Endl;
-   fLogger << "vector are assumed to be Gaussian distributed, and the inverse of" << Endl;
-   fLogger << "the covariance matrix is the H-Matrix. A multivariate chi-squared" << Endl;
-   fLogger << "estimator is built that exploits differences in the mean values of" << Endl;
-   fLogger << "the vector elements between the two classes for the purpose of" << Endl;
-   fLogger << "discrimination." << Endl;
-   fLogger << Endl;
-   fLogger << Tools::Color("bold") << "--- Performance optimisation:" << Tools::Color("reset") << Endl;
-   fLogger << Endl;
-   fLogger << "The TMVA implementation of the H-Matrix classifier has been shown" << Endl;
-   fLogger << "to underperform in comparison with the corresponding Fisher discriminant," << Endl;
-   fLogger << "when using similar assumptions and complexity. Its use is therefore" << Endl;
-   fLogger << "depreciated. Only in cases where the background model is strongly" << Endl;
-   fLogger << "non-Gaussian, H-Matrix may perform better than Fisher. In such" << Endl;
-   fLogger << "occurrences the user is advised to employ non-linear classifiers. " << Endl;
-   fLogger << Endl;
-   fLogger << Tools::Color("bold") << "--- Performance tuning via configuration options:" << Tools::Color("reset") << Endl;
-   fLogger << Endl;
-   fLogger << "<None>" << Endl;
+   Log() << Endl;
+   Log() << gTools().Color("bold") << "--- Short description:" << gTools().Color("reset") << Endl;
+   Log() << Endl;
+   Log() << "The H-Matrix classifier discriminates one class (signal) of a feature" << Endl;
+   Log() << "vector from another (background). The correlated elements of the" << Endl;
+   Log() << "vector are assumed to be Gaussian distributed, and the inverse of" << Endl;
+   Log() << "the covariance matrix is the H-Matrix. A multivariate chi-squared" << Endl;
+   Log() << "estimator is built that exploits differences in the mean values of" << Endl;
+   Log() << "the vector elements between the two classes for the purpose of" << Endl;
+   Log() << "discrimination." << Endl;
+   Log() << Endl;
+   Log() << gTools().Color("bold") << "--- Performance optimisation:" << gTools().Color("reset") << Endl;
+   Log() << Endl;
+   Log() << "The TMVA implementation of the H-Matrix classifier has been shown" << Endl;
+   Log() << "to underperform in comparison with the corresponding Fisher discriminant," << Endl;
+   Log() << "when using similar assumptions and complexity. Its use is therefore" << Endl;
+   Log() << "depreciated. Only in cases where the background model is strongly" << Endl;
+   Log() << "non-Gaussian, H-Matrix may perform better than Fisher. In such" << Endl;
+   Log() << "occurrences the user is advised to employ non-linear classifiers. " << Endl;
+   Log() << Endl;
+   Log() << gTools().Color("bold") << "--- Performance tuning via configuration options:" << gTools().Color("reset") << Endl;
+   Log() << Endl;
+   Log() << "None" << Endl;
 }
