@@ -1,4 +1,4 @@
-// @(#)root/tmva $Id: ModulekNN.cxx 21630 2008-01-10 19:40:44Z brun $
+// @(#)root/tmva $Id: ModulekNN.cxx 35738 2010-09-26 09:17:57Z stelzer $
 // Author: Rustem Ospanov 
 
 /**********************************************************************************
@@ -23,6 +23,8 @@
  * (http://tmva.sourceforge.net/LICENSE)                                          *
  **********************************************************************************/
 
+#include "TMVA/ModulekNN.h"
+
 // C++
 #include <assert.h>
 #include <iomanip>
@@ -33,8 +35,7 @@
 #include "TMath.h"
 
 // TMVA
-#include "TMVA/ModulekNN.h"
-
+#include "TMVA/MsgLogger.h"
  
 //-------------------------------------------------------------------------------------------
 TMVA::kNN::Event::Event() 
@@ -54,6 +55,16 @@ TMVA::kNN::Event::Event(const VarVec &var, const Double_t weight, const Short_t
    // constructor
 }
 
+//-------------------------------------------------------------------------------------------
+TMVA::kNN::Event::Event(const VarVec &var, const Double_t weight, const Short_t type, const VarVec &tvec)
+   :fVar(var),
+    fTgt(tvec),
+    fWeight(weight),
+    fType(type)
+{
+   // constructor
+}
+
 //-------------------------------------------------------------------------------------------
 TMVA::kNN::Event::~Event()
 {
@@ -79,6 +90,24 @@ TMVA::kNN::VarType TMVA::kNN::Event::GetDist(const Event &other) const
    return sum;
 }
 
+//-------------------------------------------------------------------------------------------
+void TMVA::kNN::Event::SetTargets(const VarVec &tvec)
+{
+   fTgt = tvec;
+}
+
+//-------------------------------------------------------------------------------------------
+const TMVA::kNN::VarVec& TMVA::kNN::Event::GetTargets() const
+{
+   return fTgt;
+}
+
+//-------------------------------------------------------------------------------------------
+const TMVA::kNN::VarVec& TMVA::kNN::Event::GetVars() const
+{
+   return fVar;
+}
+
 //-------------------------------------------------------------------------------------------
 void TMVA::kNN::Event::Print() const
 {
@@ -90,6 +119,7 @@ void TMVA::kNN::Event::Print() const
 void TMVA::kNN::Event::Print(std::ostream& os) const
 {
    // print
+   Int_t dp = os.precision();
    os << "Event: ";
    for (UInt_t ivar = 0; ivar != GetNVar(); ++ivar) {
       if (ivar == 0) {
@@ -98,7 +128,7 @@ void TMVA::kNN::Event::Print(std::ostream& os) const
       else {
          os << ", ";
       }
-      
+
       os << std::setfill(' ') << std::setw(5) << std::setprecision(3) << GetVar(ivar);
    }
 
@@ -108,32 +138,40 @@ void TMVA::kNN::Event::Print(std::ostream& os) const
    else {
       os << " no variables";
    }
+   os << std::setprecision(dp);
 }
 
 //-------------------------------------------------------------------------------------------
 std::ostream& TMVA::kNN::operator<<(std::ostream& os, const TMVA::kNN::Event& event)
-{ 
+{
    // streamer
    event.Print(os);
    return os;
 }
 
+
+
+
+
+TRandom3 TMVA::kNN::ModulekNN::fgRndm(1);
+
 //-------------------------------------------------------------------------------------------
-TMVA::kNN::ModulekNN::ModulekNN() 
+TMVA::kNN::ModulekNN::ModulekNN()
    :fDimn(0),
     fTree(0),
-    fLogger( "ModulekNN" )
+    fLogger( new MsgLogger("ModulekNN") )
 {
    // default constructor
 }
 
 //-------------------------------------------------------------------------------------------
 TMVA::kNN::ModulekNN::~ModulekNN()
-{ 
+{
    // destructor
    if (fTree) {
-      delete fTree;
+      delete fTree; fTree = 0;
    }
+   delete fLogger;
 }
 
 //-------------------------------------------------------------------------------------------
@@ -158,7 +196,7 @@ void TMVA::kNN::ModulekNN::Add(const Event &event)
 {   
    // add an event to tree
    if (fTree) {
-      fLogger << kFATAL << "<Add> Cannot add event: tree is already built" << Endl;
+      Log() << kFATAL << "<Add> Cannot add event: tree is already built" << Endl;
       return;      
    }
 
@@ -166,7 +204,7 @@ void TMVA::kNN::ModulekNN::Add(const Event &event)
       fDimn = event.GetNVar();
    }
    else if (fDimn != event.GetNVar()) {
-      fLogger << kFATAL << "ModulekNN::Add() - number of dimension does not match previous events" << Endl;
+      Log() << kFATAL << "ModulekNN::Add() - number of dimension does not match previous events" << Endl;
       return;
    }
 
@@ -190,7 +228,7 @@ Bool_t TMVA::kNN::ModulekNN::Fill(const UShort_t odepth, const UInt_t ifrac, con
 {
    // fill the tree
    if (fTree) {
-      fLogger << kFATAL << "ModulekNN::Fill - tree already have been created" << Endl;
+      Log() << kFATAL << "ModulekNN::Fill - tree has already been created" << Endl;
       return kFALSE;
    }   
 
@@ -204,7 +242,7 @@ Bool_t TMVA::kNN::ModulekNN::Fill(const UShort_t odepth, const UInt_t ifrac, con
          }
       }
       
-      fLogger << kINFO << "<Fill> Will trim all event types to " << min << " events" << Endl;
+      Log() << kINFO << "<Fill> Will trim all event types to " << min << " events" << Endl;
       
       fCount.clear();
       fVar.clear();
@@ -222,15 +260,15 @@ Bool_t TMVA::kNN::ModulekNN::Fill(const UShort_t odepth, const UInt_t ifrac, con
          else {
             continue;
          }
-   
+
          for (UInt_t d = 0; d < fDimn; ++d) {
             fVar[d].push_back(event->GetVar(d));
-         }   
+         }
 
          evec.push_back(*event);
       }
 
-      fLogger << kINFO << "<Fill> Erased " << fEvent.size() - evec.size() << " events" << Endl;
+      Log() << kINFO << "<Fill> Erased " << fEvent.size() - evec.size() << " events" << Endl;
       
       fEvent = evec;
    }
@@ -258,9 +296,9 @@ Bool_t TMVA::kNN::ModulekNN::Fill(const UShort_t odepth, const UInt_t ifrac, con
    // all child nodes. If odepth = 0 then split variable 0
    // at the median (in half) and return it as root node
    fTree = Optimize(odepth);
-
+   
    if (!fTree) {
-      fLogger << kFATAL << "ModulekNN::Fill() - failed to create tree" << Endl;
+      Log() << kFATAL << "ModulekNN::Fill() - failed to create tree" << Endl;
       return kFALSE;      
    }      
    
@@ -277,7 +315,7 @@ Bool_t TMVA::kNN::ModulekNN::Fill(const UShort_t odepth, const UInt_t ifrac, con
    }
    
    for (std::map<Short_t, UInt_t>::const_iterator it = fCount.begin(); it != fCount.end(); ++it) {
-      fLogger << kINFO << "<Fill> Class " << it->first << " has " << std::setw(8) 
+      Log() << kINFO << "<Fill> Class " << it->first << " has " << std::setw(8) 
               << it->second << " events" << Endl;
    }
    
@@ -285,7 +323,7 @@ Bool_t TMVA::kNN::ModulekNN::Fill(const UShort_t odepth, const UInt_t ifrac, con
 }
 
 //-------------------------------------------------------------------------------------------
-Bool_t TMVA::kNN::ModulekNN::Find(Event event, const UInt_t nfind) const
+Bool_t TMVA::kNN::ModulekNN::Find(Event event, const UInt_t nfind, const std::string &option) const
 {  
    // find in tree
    // if tree has been filled then search for nfind closest events 
@@ -293,15 +331,15 @@ Bool_t TMVA::kNN::ModulekNN::Find(Event event, const UInt_t nfind) const
    // using previsouly computed width of variable distribution
 
    if (!fTree) {
-      fLogger << kFATAL << "ModulekNN::Find() - tree has not been filled" << Endl;
+      Log() << kFATAL << "ModulekNN::Find() - tree has not been filled" << Endl;
       return kFALSE;
    }
    if (fDimn != event.GetNVar()) {
-      fLogger << kFATAL << "ModulekNN::Find() - number of dimension does not match training events" << Endl;
+      Log() << kFATAL << "ModulekNN::Find() - number of dimension does not match training events" << Endl;
       return kFALSE;
    }
    if (nfind < 1) {
-      fLogger << kFATAL << "ModulekNN::Find() - requested 0 nearest neighbors" << Endl;
+      Log() << kFATAL << "ModulekNN::Find() - requested 0 nearest neighbors" << Endl;
       return kFALSE;
    }
 
@@ -315,8 +353,19 @@ Bool_t TMVA::kNN::ModulekNN::Find(Event event, const UInt_t nfind) const
    fkNNEvent = event;
    fkNNList.clear();
    
-   // recursive kd-tree search for nfind-nearest neighbors
-   kNN::Find<kNN::Event>(fkNNList, fTree, event, nfind);
+   if(option.find("weight") != std::string::npos)
+   {
+      // recursive kd-tree search for nfind-nearest neighbors
+      // use event weight to find all nearest events
+      // that have sum of weights >= nfind
+      kNN::Find<kNN::Event>(fkNNList, fTree, event, Double_t(nfind), 0.0);
+   }
+   else
+   {
+      // recursive kd-tree search for nfind-nearest neighbors
+      // count nodes and do not use event weight
+      kNN::Find<kNN::Event>(fkNNList, fTree, event, nfind);      
+   }
 
    return kTRUE;
 }
@@ -357,7 +406,7 @@ Bool_t TMVA::kNN::ModulekNN::Find(const UInt_t nfind, const std::string &option)
          const VarType width = max - min;
          
          if (width < 0.0 || width > 0.0) {
-            dvec.push_back(min + width*double(std::rand())/double(RAND_MAX));
+            dvec.push_back(min + width*fgRndm.Rndm());
          }
          else {
             return kFALSE;
@@ -366,7 +415,7 @@ Bool_t TMVA::kNN::ModulekNN::Find(const UInt_t nfind, const std::string &option)
 
       const Event event(dvec, 1.0, etype);
       
-      Find(event, nfind);  
+      Find(event, nfind);
       
       return kTRUE;
    }
@@ -382,36 +431,36 @@ TMVA::kNN::Node<TMVA::kNN::Event>* TMVA::kNN::ModulekNN::Optimize(const UInt_t o
    // into 2^odepth parts
 
    if (fVar.empty() || fDimn != fVar.size()) {
-      fLogger << kWARNING << "<Optimize> Cannot build a tree" << Endl;
+      Log() << kWARNING << "<Optimize> Cannot build a tree" << Endl;
       return 0;
    }
 
    const UInt_t size = (fVar.begin()->second).size();
    if (size < 1) {
-      fLogger << kWARNING << "<Optimize> Cannot build a tree without events" << Endl;
+      Log() << kWARNING << "<Optimize> Cannot build a tree without events" << Endl;
       return 0;
    }
 
    VarMap::const_iterator it = fVar.begin();
    for (; it != fVar.end(); ++it) {
       if ((it->second).size() != size) {
-         fLogger << kWARNING << "<Optimize> # of variables doesn't match between dimensions" << Endl;
+         Log() << kWARNING << "<Optimize> # of variables doesn't match between dimensions" << Endl;
          return 0;
       }
    }
 
    if (double(fDimn*size) < TMath::Power(2.0, double(odepth))) {
-      fLogger << kWARNING << "<Optimize> Optimization depth exceeds number of events" << Endl;
+      Log() << kWARNING << "<Optimize> Optimization depth exceeds number of events" << Endl;
       return 0;      
    }   
 
-   fLogger << kINFO << "Optimizing tree for " << fDimn << " variables with " << size << " values" << Endl;
+   Log() << kINFO << "Optimizing tree for " << fDimn << " variables with " << size << " values" << Endl;
 
    std::vector<Node<Event> *> pvec, cvec;
 
    it = fVar.find(0);
    if (it == fVar.end() || (it->second).size() < 2) {
-      fLogger << kWARNING << "<Optimize> Missing 0 variable" << Endl;
+      Log() << kWARNING << "<Optimize> Missing 0 variable" << Endl;
       return 0;
    }
 
@@ -426,14 +475,14 @@ TMVA::kNN::Node<TMVA::kNN::Event>* TMVA::kNN::ModulekNN::Optimize(const UInt_t o
       
       VarMap::const_iterator vit = fVar.find(mod);
       if (vit == fVar.end()) {
-         fLogger << kFATAL << "Missing " << mod << " variable" << Endl;
+         Log() << kFATAL << "Missing " << mod << " variable" << Endl;
          return 0;
       }
       const std::vector<Double_t> &dvec = vit->second;
 
       if (dvec.size() < 2) {
-         fLogger << kFATAL << "Missing " << mod << " variable" << Endl;
-         return 0;   
+         Log() << kFATAL << "Missing " << mod << " variable" << Endl;
+         return 0;
       }
       
       UInt_t ichild = 1;
@@ -478,22 +527,22 @@ void TMVA::kNN::ModulekNN::ComputeMetric(const UInt_t ifrac)
       return;
    }
    if (ifrac > 100) {
-      fLogger << kFATAL << "ModulekNN::ComputeMetric - fraction can not exceed 100%" << Endl;
+      Log() << kFATAL << "ModulekNN::ComputeMetric - fraction can not exceed 100%" << Endl;
       return;
    }   
    if (!fVarScale.empty()) {
-      fLogger << kFATAL << "ModulekNN::ComputeMetric - metric is already computed" << Endl;
+      Log() << kFATAL << "ModulekNN::ComputeMetric - metric is already computed" << Endl;
       return;
    }
    if (fEvent.size() < 100) {
-      fLogger << kFATAL << "ModulekNN::ComputeMetric - number of events is too small" << Endl;
+      Log() << kFATAL << "ModulekNN::ComputeMetric - number of events is too small" << Endl;
       return;      
    }
 
    const UInt_t lfrac = (100 - ifrac)/2;
    const UInt_t rfrac = 100 - (100 - ifrac)/2;
 
-   fLogger << kINFO << "Computing scale factor for 1d distributions: " 
+   Log() << kINFO << "Computing scale factor for 1d distributions: " 
            << "(ifrac, bottom, top) = (" << ifrac << "%, " << lfrac << "%, " << rfrac << "%)" << Endl;   
 
    fVarScale.clear();
@@ -529,12 +578,12 @@ void TMVA::kNN::ModulekNN::ComputeMetric(const UInt_t ifrac)
       const Double_t rpos = *end_it;
       
       if (!(lpos < rpos)) {
-         fLogger << kFATAL << "ModulekNN::ComputeMetric() - min value is greater than max value" << Endl;
-         continue;   
+         Log() << kFATAL << "ModulekNN::ComputeMetric() - min value is greater than max value" << Endl;
+         continue;
       }
       
       // Rustem: please find a solution that does not use distance (it does not exist on solaris)
-      //       fLogger << kINFO << "Variable " << vit->first 
+      //       Log() << kINFO << "Variable " << vit->first 
       //               << " included " << distance(beg_it, end_it) + 1
       //               << " events: width = " << std::setfill(' ') << std::setw(5) << std::setprecision(3) << rpos - lpos
       //               << ", (min, max) = (" << std::setfill(' ') << std::setw(5) << std::setprecision(3) << lpos 
@@ -550,7 +599,7 @@ void TMVA::kNN::ModulekNN::ComputeMetric(const UInt_t ifrac)
       
       for (UInt_t ivar = 0; ivar < fDimn; ++ivar) {
          fVar[ivar].push_back(fEvent[ievent].GetVar(ivar));
-      }   
+      }
    }
 }
 
@@ -565,7 +614,7 @@ const TMVA::kNN::Event TMVA::kNN::ModulekNN::Scale(const Event &event) const
    }
 
    if (event.GetNVar() != fVarScale.size()) {
-      fLogger << kFATAL << "ModulekNN::Scale() - mismatched metric and event size" << Endl;
+      Log() << kFATAL << "ModulekNN::Scale() - mismatched metric and event size" << Endl;
       return event;
    }
 
@@ -574,7 +623,7 @@ const TMVA::kNN::Event TMVA::kNN::ModulekNN::Scale(const Event &event) const
    for (UInt_t ivar = 0; ivar < event.GetNVar(); ++ivar) {
       std::map<int, Double_t>::const_iterator fit = fVarScale.find(ivar);
       if (fit == fVarScale.end()) {
-         fLogger << kFATAL << "ModulekNN::Scale() - failed to find scale for " << ivar << Endl;
+         Log() << kFATAL << "ModulekNN::Scale() - failed to find scale for " << ivar << Endl;
          continue;
       }
       
@@ -582,11 +631,11 @@ const TMVA::kNN::Event TMVA::kNN::ModulekNN::Scale(const Event &event) const
          vvec[ivar] = event.GetVar(ivar)/fit->second;
       }
       else {
-         fLogger << kFATAL << "Variable " << ivar << " has zero width" << Endl;
+         Log() << kFATAL << "Variable " << ivar << " has zero width" << Endl;
       }
    }
 
-   return Event(vvec, event.GetWeight(), event.GetType());
+   return Event(vvec, event.GetWeight(), event.GetType(), event.GetTargets());
 }
 
 //-------------------------------------------------------------------------------------------
@@ -613,7 +662,7 @@ void TMVA::kNN::ModulekNN::Print(ostream &os) const
       os << ++count << ": " << it->second << ": " << it->first->GetEvent() << std::endl;
       
       const Event &event = it->first->GetEvent();
-      for (UShort_t ivar = 0; ivar < event.GetNVar(); ++ivar) {   
+      for (UShort_t ivar = 0; ivar < event.GetNVar(); ++ivar) {
          if (min.find(ivar) == min.end()) {
             min[ivar] = event.GetVar(ivar);
          }
@@ -633,7 +682,7 @@ void TMVA::kNN::ModulekNN::Print(ostream &os) const
    if (min.size() == max.size()) {
       for (std::map<Short_t, Double_t>::const_iterator mit = min.begin(); mit != min.end(); ++mit) {
          const Short_t i = mit->first;
-         fLogger << kINFO << "(var, min, max) = (" << i << "," << min[i] << ", " << max[i] << ")" << Endl;
+         Log() << kINFO << "(var, min, max) = (" << i << "," << min[i] << ", " << max[i] << ")" << Endl;
       }
    }
    
