@@ -1,4 +1,4 @@
-// @(#)root/tmva $Id: RuleEnsemble.cxx 21630 2008-01-10 19:40:44Z brun $
+// @(#)root/tmva $Id: RuleEnsemble.cxx 41891 2011-11-10 22:46:31Z pcanal $
 // Author: Andreas Hoecker, Joerg Stelzer, Fredrik Tegenfeldt, Helge Voss
 
 /**********************************************************************************
@@ -17,9 +17,9 @@
  *      Helge Voss         <Helge.Voss@cern.ch>         - MPI-KP Heidelberg, GER  *
  *                                                                                *
  * Copyright (c) 2005:                                                            *
- *      CERN, Switzerland                                                         * 
+ *      CERN, Switzerland                                                         *
  *      Iowa State U.                                                             *
- *      MPI-K Heidelberg, Germany                                                 * 
+ *      MPI-K Heidelberg, Germany                                                 *
  *                                                                                *
  * Redistribution and use in source and binary forms, with or without             *
  * modification, are permitted according to the terms listed in LICENSE           *
@@ -28,23 +28,36 @@
 
 #include <algorithm>
 #include <list>
+#include <cstdlib>
+#include <iomanip>
 
-#include "Riostream.h"
-#include "TRandom.h"
+#include "TRandom3.h"
 #include "TH1F.h"
 #include "TMVA/RuleEnsemble.h"
 #include "TMVA/RuleFit.h"
 #include "TMVA/MethodRuleFit.h"
+#include "TMVA/Tools.h"
 
 //_______________________________________________________________________
 TMVA::RuleEnsemble::RuleEnsemble( RuleFit *rf )
-   : fLearningModel    ( kFull )
-   , fLinQuantile      ( 0.025 ) // default quantile for killing outliers in linear terms
-   , fAverageSupport   ( 0.8 )
-   , fAverageRuleSigma ( 0.4 )  // default value - used if only linear model is chosen
-   , fRuleMinDist      ( 1e-3 ) // closest allowed 'distance' between two rules
-   , fRuleMapEvents    ( 0 )
-   , fLogger( "RuleFit" )
+   : fLearningModel   ( kFull )
+   , fImportanceCut   ( 0 )
+   , fLinQuantile     ( 0.025 ) // default quantile for killing outliers in linear terms
+   , fOffset          ( 0 )
+   , fAverageSupport  ( 0.8 )
+   , fAverageRuleSigma( 0.4 )  // default value - used if only linear model is chosen
+   , fRuleFSig        ( 0 )
+   , fRuleNCave       ( 0 )
+   , fRuleNCsig       ( 0 )
+   , fRuleMinDist     ( 1e-3 ) // closest allowed 'distance' between two rules
+   , fNRulesGenerated ( 0 )
+   , fEvent           ( 0 )
+   , fEventCacheOK    ( true )
+   , fRuleMapOK       ( true )
+   , fRuleMapInd0     ( 0 )
+   , fRuleMapInd1     ( 0 )
+   , fRuleMapEvents   ( 0 )
+   , fLogger( new MsgLogger("RuleFit") )
 {
    // constructor
    Initialize( rf );
@@ -53,8 +66,10 @@ TMVA::RuleEnsemble::RuleEnsemble( RuleFit *rf )
 //_______________________________________________________________________
 TMVA::RuleEnsemble::RuleEnsemble( const RuleEnsemble& other )
    : fAverageSupport   ( 1 )
+   , fEvent(0)
    , fRuleMapEvents(0)
-   , fLogger( "RuleFit" )
+   , fRuleFit(0)
+   , fLogger( new MsgLogger("RuleFit") )
 {
    // copy constructor
    Copy( other );
@@ -62,8 +77,26 @@ TMVA::RuleEnsemble::RuleEnsemble( const RuleEnsemble& other )
 
 //_______________________________________________________________________
 TMVA::RuleEnsemble::RuleEnsemble()
-   : fRuleMapEvents(0)
-   , fLogger( "RuleFit" )
+   : fLearningModel     ( kFull )
+   , fImportanceCut   ( 0 )
+   , fLinQuantile     ( 0.025 ) // default quantile for killing outliers in linear terms
+   , fOffset          ( 0 )
+   , fImportanceRef   ( 1.0 )
+   , fAverageSupport  ( 0.8 )
+   , fAverageRuleSigma( 0.4 )  // default value - used if only linear model is chosen
+   , fRuleFSig        ( 0 )
+   , fRuleNCave       ( 0 )
+   , fRuleNCsig       ( 0 )
+   , fRuleMinDist     ( 1e-3 ) // closest allowed 'distance' between two rules
+   , fNRulesGenerated ( 0 )
+   , fEvent           ( 0 )
+   , fEventCacheOK    ( true )
+   , fRuleMapOK       ( true )
+   , fRuleMapInd0     ( 0 )
+   , fRuleMapInd1     ( 0 )
+   , fRuleMapEvents   ( 0 )
+   , fRuleFit         ( 0 )
+   , fLogger( new MsgLogger("RuleFit") )
 {
    // constructor
 }
@@ -76,6 +109,7 @@ TMVA::RuleEnsemble::~RuleEnsemble()
       delete *itrRule;
    }
    // NOTE: Should not delete the histos fLinPDFB/S since they are delete elsewhere
+   delete fLogger;
 }
 
 //_______________________________________________________________________
@@ -99,6 +133,12 @@ void TMVA::RuleEnsemble::Initialize( const RuleFit *rf )
    }
 }
 
+//_______________________________________________________________________
+void TMVA::RuleEnsemble::SetMsgType( EMsgType t ) {
+   fLogger->SetMinType(t);
+}
+
+
 //_______________________________________________________________________
 const TMVA::MethodRuleFit*  TMVA::RuleEnsemble::GetMethodRuleFit() const
 {
@@ -171,8 +211,8 @@ void TMVA::RuleEnsemble::SetCoefficients( const std::vector< Double_t > & v )
 
    UInt_t nrules = fRules.size();
    if (v.size()!=nrules) {
-      fLogger << kFATAL << "<SetCoefficients> - BUG TRAP - input vector worng size! It is = " << v.size()
-              << " when it should be = " << nrules << Endl;
+      Log() << kFATAL << "<SetCoefficients> - BUG TRAP - input vector worng size! It is = " << v.size()
+            << " when it should be = " << nrules << Endl;
    }
    for (UInt_t i=0; i<nrules; i++) {
       fRules[i]->SetCoefficient(v[i]);
@@ -213,11 +253,11 @@ void TMVA::RuleEnsemble::RemoveSimilarRules()
 {
    // remove rules that behave similar 
    
-   fLogger << kVERBOSE << "Removing similar rules; distance = " << fRuleMinDist << Endl;
+   Log() << kVERBOSE << "Removing similar rules; distance = " << fRuleMinDist << Endl;
 
    UInt_t nrulesIn = fRules.size();
    TMVA::Rule *first, *second;
-   std::vector<bool> removeMe( nrulesIn,false );
+   std::vector< Char_t > removeMe( nrulesIn,false );  // <--- stores boolean
 
    Int_t nrem = 0;
    Int_t remind=-1;
@@ -233,7 +273,8 @@ void TMVA::RuleEnsemble::RemoveSimilarRules()
                if (equal) {
                   r = gRandom->Rndm();
                   remind = (r>0.5 ? k:i); // randomly select rule
-               } else {
+               } 
+               else {
                   remind = -1;
                }
 
@@ -252,15 +293,18 @@ void TMVA::RuleEnsemble::RemoveSimilarRules()
    for (UInt_t i=0; i<nrulesIn; i++) {
       if (removeMe[i]) {
          theRule = fRules[ind];
+#if _MSC_VER >= 1400
+         fRules.erase( std::vector<Rule *>::iterator(&fRules[ind], &fRules) );
+#else
          fRules.erase( std::vector<Rule *>::iterator(&fRules[ind]) );
+#endif
          delete theRule;
          ind--;
-      } else {
-      }
+      } 
       ind++;
    }
    UInt_t nrulesOut = fRules.size();
-   fLogger << kVERBOSE << "Removed " << nrulesIn - nrulesOut << " out of " << nrulesIn << " rules" << Endl;
+   Log() << kVERBOSE << "Removed " << nrulesIn - nrulesOut << " out of " << nrulesIn << " rules" << Endl;
 }
 
 //_______________________________________________________________________
@@ -270,7 +314,7 @@ void TMVA::RuleEnsemble::CleanupRules()
 
    UInt_t nrules   = fRules.size();
    if (nrules==0) return;
-   fLogger << kVERBOSE << "Removing rules with relative importance < " << fImportanceCut << Endl;
+   Log() << kVERBOSE << "Removing rules with relative importance < " << fImportanceCut << Endl;
    if (fImportanceCut<=0) return;
    //
    // Mark rules to be removed
@@ -280,14 +324,18 @@ void TMVA::RuleEnsemble::CleanupRules()
    for (UInt_t i=0; i<nrules; i++) {
       if (fRules[ind]->GetRelImportance()<fImportanceCut) {
          therule = fRules[ind];
+#if _MSC_VER >= 1400
+         fRules.erase( std::vector<Rule *>::iterator(&fRules[ind], &fRules) );
+#else
          fRules.erase( std::vector<Rule *>::iterator(&fRules[ind]) );
+#endif
          delete therule;
          ind--;
       } 
       ind++;
    }
-   fLogger << kINFO << "Removed " << nrules-ind << " out of a total of " << nrules
-           << " rules with importance < " << fImportanceCut << Endl;
+   Log() << kINFO << "Removed " << nrules-ind << " out of a total of " << nrules
+         << " rules with importance < " << fImportanceCut << Endl;
 }
 
 //_______________________________________________________________________
@@ -297,7 +345,7 @@ void TMVA::RuleEnsemble::CleanupLinear()
 
    UInt_t nlin = fLinNorm.size();
    if (nlin==0) return;
-   fLogger << kVERBOSE << "Removing linear terms with relative importance < " << fImportanceCut << Endl;
+   Log() << kVERBOSE << "Removing linear terms with relative importance < " << fImportanceCut << Endl;
    //
    fLinTermOK.clear();
    for (UInt_t i=0; i<nlin; i++) {
@@ -309,7 +357,7 @@ void TMVA::RuleEnsemble::CleanupLinear()
 void TMVA::RuleEnsemble::CalcRuleSupport()
 {
    // calculate the support for all rules
-   fLogger << kVERBOSE << "Evaluating Rule support" << Endl;
+   Log() << kVERBOSE << "Evaluating Rule support" << Endl;
    Double_t s,t,stot,ttot,ssb;
    Double_t ssig, sbkg, ssum;
    Int_t indrule=0;
@@ -330,7 +378,7 @@ void TMVA::RuleEnsemble::CalcRuleSupport()
             if ((*itrRule)->EvalEvent( *(*itrEvent) )) {
                ew = (*itrEvent)->GetWeight();
                s += ew;
-               if ((*itrEvent)->IsSignal()) ssig += ew;
+               if (GetMethodRuleFit()->DataInfo().IsSignal(*itrEvent)) ssig += ew;
                else                         sbkg += ew;
             }
          }
@@ -350,8 +398,8 @@ void TMVA::RuleEnsemble::CalcRuleSupport()
       }
       fAverageSupport   = stot/nrules;
       fAverageRuleSigma = TMath::Sqrt(fAverageSupport*(1.0-fAverageSupport));
-      fLogger << kVERBOSE << "Standard deviation of support = " << fAverageRuleSigma << Endl;
-      fLogger << kVERBOSE << "Average rule support          = " << fAverageSupport   << Endl;
+      Log() << kVERBOSE << "Standard deviation of support = " << fAverageRuleSigma << Endl;
+      Log() << kVERBOSE << "Average rule support          = " << fAverageSupport   << Endl;
    }
 }
 
@@ -427,10 +475,10 @@ void TMVA::RuleEnsemble::CalcVarImportance()
    //
    // Calculates variable importance using eq (35) in RuleFit paper by Friedman et.al
    //
-   fLogger << kVERBOSE << "Compute variable importance" << Endl;
+   Log() << kVERBOSE << "Compute variable importance" << Endl;
    Double_t rimp;
    UInt_t nrules = fRules.size();
-   if (GetMethodBase()==0) fLogger << kFATAL << "RuleEnsemble::CalcVarImportance() - should not be here!" << Endl;
+   if (GetMethodBase()==0) Log() << kFATAL << "RuleEnsemble::CalcVarImportance() - should not be here!" << Endl;
    UInt_t nvars  = GetMethodBase()->GetNvar();
    UInt_t nvarsUsed;
    Double_t rimpN;
@@ -441,7 +489,7 @@ void TMVA::RuleEnsemble::CalcVarImportance()
          rimp = fRules[ind]->GetImportance();
          nvarsUsed = fRules[ind]->GetNumVarsUsed();
          if (nvarsUsed<1)
-            fLogger << kFATAL << "<CalcVarImportance> Variables for importance calc!!!??? A BUG!" << Endl;
+            Log() << kFATAL << "<CalcVarImportance> Variables for importance calc!!!??? A BUG!" << Endl;
          rimpN = (nvarsUsed > 0 ? rimp/nvarsUsed:0.0);
          for ( UInt_t iv=0; iv<nvars; iv++ ) {
             if (fRules[ind]->ContainsVariable(iv)) {
@@ -501,10 +549,10 @@ void TMVA::RuleEnsemble::MakeRules( const std::vector< const DecisionTree *> & f
    Double_t sumnendn=0;
    Double_t sumn2=0;
    //
-   UInt_t prevs;
+   // UInt_t prevs;
    UInt_t ntrees = forest.size();
    for ( UInt_t ind=0; ind<ntrees; ind++ ) {
-      prevs = fRules.size();
+      // prevs = fRules.size();
       MakeRulesFromTree( forest[ind] );
       nrules = CalcNRules( forest[ind] );
       nendn = (nrules/2) + 1;
@@ -513,22 +561,22 @@ void TMVA::RuleEnsemble::MakeRules( const std::vector< const DecisionTree *> & f
       nrulesCheck += nrules;
    }
    Double_t nmean = sumnendn/ntrees;
-   Double_t nsigm = TMath::Sqrt( Tools::ComputeVariance(sumn2,sumnendn,ntrees) );
+   Double_t nsigm = TMath::Sqrt( gTools().ComputeVariance(sumn2,sumnendn,ntrees) );
    Double_t ndev = 2.0*(nmean-2.0-nsigm)/(nmean-2.0+nsigm);
    //
-   fLogger << kVERBOSE << "Average number of end nodes per tree   = " << nmean << Endl;
-   if (ntrees>1) fLogger << kVERBOSE << "sigma of ditto ( ~= mean-2 ?)          = "
-                         << nsigm
-                         << Endl;
-   fLogger << kVERBOSE << "Deviation from exponential model       = " << ndev      << Endl;
-   fLogger << kVERBOSE << "Corresponds to L (eq. 13, RuleFit ppr) = " << nmean << Endl;
+   Log() << kVERBOSE << "Average number of end nodes per tree   = " << nmean << Endl;
+   if (ntrees>1) Log() << kVERBOSE << "sigma of ditto ( ~= mean-2 ?)          = "
+                       << nsigm
+                       << Endl;
+   Log() << kVERBOSE << "Deviation from exponential model       = " << ndev      << Endl;
+   Log() << kVERBOSE << "Corresponds to L (eq. 13, RuleFit ppr) = " << nmean << Endl;
    // a BUG trap
    if (nrulesCheck != static_cast<Int_t>(fRules.size())) {
-     fLogger << kFATAL 
-             << "BUG! number of generated and possible rules do not match! N(rules) =  " << fRules.size() 
-             << " != " << nrulesCheck << Endl;
+      Log() << kFATAL 
+            << "BUG! number of generated and possible rules do not match! N(rules) =  " << fRules.size() 
+            << " != " << nrulesCheck << Endl;
    }
-   fLogger << kVERBOSE << "Number of generated rules: " << fRules.size() << Endl;
+   Log() << kVERBOSE << "Number of generated rules: " << fRules.size() << Endl;
 
    // save initial number of rules
    fNRulesGenerated = fRules.size();
@@ -550,7 +598,7 @@ void TMVA::RuleEnsemble::MakeLinearTerms()
 
    const std::vector<Event *> *events = GetTrainingEvents();
    UInt_t neve  = events->size();
-   UInt_t nvars = ((*events)[0])->GetNVars(); // Event -> GetNVars();
+   UInt_t nvars = ((*events)[0])->GetNVariables(); // Event -> GetNVariables();
    Double_t val,ew;
    typedef std::pair< Double_t, Int_t> dataType;
    typedef std::pair< Double_t, dataType > dataPoint;
@@ -565,8 +613,8 @@ void TMVA::RuleEnsemble::MakeLinearTerms()
    for (UInt_t i=0; i<neve; i++) {
       ew   = ((*events)[i])->GetWeight();
       for (UInt_t v=0; v<nvars; v++) {
-         val = ((*events)[i])->GetVal(v);
-         vardata[v].push_back( dataPoint( val, dataType(ew,((*events)[i])->Type()) ) );
+         val = ((*events)[i])->GetValue(v);
+         vardata[v].push_back( dataPoint( val, dataType(ew,((*events)[i])->GetClass()) ) );
       }
    }
    //
@@ -624,7 +672,7 @@ void TMVA::RuleEnsemble::MakeLinearTerms()
       //
       Int_t type;
       const Double_t w = 1.0/fRuleFit->GetNEveEff();
-      for (UInt_t ie=0; ie<neve; ie++) {
+      for (ie=0; ie<neve; ie++) {
          val  = vardata[v][ie].first;
          ew   = vardata[v][ie].second.first;
          type = vardata[v][ie].second.second;
@@ -682,15 +730,18 @@ Double_t TMVA::RuleEnsemble::PdfRule( Double_t & nsig, Double_t & ntot  ) const
    Double_t sump  = 0;
    Double_t sumok = 0;
    Double_t sumz  = 0;
+   Double_t ssb;
+   Double_t neve;
    //
    UInt_t nrules = fRules.size();
    for (UInt_t ir=0; ir<nrules; ir++) {
-      if(fEventRuleVal[ir]) {
-         Double_t ssb = GetRulesConst(ir)->GetSSB(); // S/(S+B) is evaluated in CalcRuleSupport() using ALL training events
-         Double_t neve = GetRulesConst(ir)->GetSSBNeve(); // number of events accepted by the rule
+      if (fEventRuleVal[ir]>0) {
+         ssb = fEventRuleVal[ir]*GetRulesConst(ir)->GetSSB(); // S/(S+B) is evaluated in CalcRuleSupport() using ALL training events
+         neve = GetRulesConst(ir)->GetSSBNeve(); // number of events accepted by the rule
          sump  += ssb*neve; // number of signal events
          sumok += neve; // total number of events passed
-      } else sumz += 1.0; // all events
+      } 
+      else sumz += 1.0; // all events
    }
 
    nsig = sump;
@@ -753,7 +804,7 @@ void TMVA::RuleEnsemble::RuleResponseStats()
    Bool_t sigRule;
    Bool_t sigTag;
    Bool_t bkgTag;
-   Bool_t noTag;
+   // Bool_t noTag;
    Bool_t sigTrue;
    Bool_t tagged;
    // Counters
@@ -783,7 +834,8 @@ void TMVA::RuleEnsemble::RuleResponseStats()
       sigRule = fRules[i]->IsSignalRule();
       if (sigRule) { // rule is a signal rule (ie s/(s+b)>0.5)
          nsig++;
-      } else {
+      } 
+      else {
          nbkg++;
       }
       // reset counters
@@ -798,8 +850,8 @@ void TMVA::RuleEnsemble::RuleResponseStats()
          tagged  = fRules[i]->EvalEvent(*eveData);
          sigTag = (tagged && sigRule);        // it's tagged as a signal
          bkgTag = (tagged && (!sigRule));     // ... as bkg
-         noTag = !(sigTag || bkgTag);         // ... not tagged
-         sigTrue = eveData->IsSignal();       // true if event is true signal
+         // noTag = !(sigTag || bkgTag);         // ... not tagged
+         sigTrue = (eveData->GetClass() == 0);       // true if event is true signal
          if (tagged) {
             ntag++;
             if (sigTag && sigTrue)  nss++;
@@ -839,7 +891,7 @@ void TMVA::RuleEnsemble::RuleStatistics()
    fRuleNCsig = 0.0;
    if (nrules>0) {
       fRuleNCave = sumNc/nrules;
-      fRuleNCsig = TMath::Sqrt(Tools::ComputeVariance(sumNc2,sumNc,nrules));
+      fRuleNCsig = TMath::Sqrt(gTools().ComputeVariance(sumNc2,sumNc,nrules));
    }
 }
 
@@ -847,19 +899,18 @@ void TMVA::RuleEnsemble::RuleStatistics()
 void TMVA::RuleEnsemble::PrintRuleGen() const
 {
    // print rule generation info
-   if (!DoRules()) return;
-   fLogger << kINFO << "-------------------RULE ENSEMBLE SUMMARY------------------------"  << Endl;
+   Log() << kINFO << "-------------------RULE ENSEMBLE SUMMARY------------------------"  << Endl;
    const MethodRuleFit *mrf = GetMethodRuleFit();
-   if (mrf) fLogger << kINFO << "Tree training method               : " << (mrf->UseBoost() ? "AdaBoost":"Random") << Endl;
-   fLogger << kINFO << "Number of events per tree          : " << fRuleFit->GetNTreeSample()    << Endl;
-   fLogger << kINFO << "Number of trees                    : " << fRuleFit->GetForest().size() << Endl;
-   fLogger << kINFO << "Number of generated rules          : " << fNRulesGenerated << Endl;
-   fLogger << kINFO << "Idem, after cleanup                : " << fRules.size() << Endl;
-   fLogger << kINFO << "Average number of cuts per rule    : " << Form("%8.2f",fRuleNCave) << Endl;
-   fLogger << kINFO << "Spread in number of cuts per rules : " << Form("%8.2f",fRuleNCsig) << Endl;
-   fLogger << kVERBOSE << "Complexity                         : " << Form("%8.2f",fRuleNCave*fRuleNCsig) << Endl;
-   fLogger << kINFO << "----------------------------------------------------------------"  << Endl;
-   fLogger << kINFO << Endl;
+   if (mrf) Log() << kINFO << "Tree training method               : " << (mrf->UseBoost() ? "AdaBoost":"Random") << Endl;
+   Log() << kINFO << "Number of events per tree          : " << fRuleFit->GetNTreeSample()    << Endl;
+   Log() << kINFO << "Number of trees                    : " << fRuleFit->GetForest().size() << Endl;
+   Log() << kINFO << "Number of generated rules          : " << fNRulesGenerated << Endl;
+   Log() << kINFO << "Idem, after cleanup                : " << fRules.size() << Endl;
+   Log() << kINFO << "Average number of cuts per rule    : " << Form("%8.2f",fRuleNCave) << Endl;
+   Log() << kINFO << "Spread in number of cuts per rules : " << Form("%8.2f",fRuleNCsig) << Endl;
+   Log() << kVERBOSE << "Complexity                         : " << Form("%8.2f",fRuleNCave*fRuleNCsig) << Endl;
+   Log() << kINFO << "----------------------------------------------------------------"  << Endl;
+   Log() << kINFO << Endl;
 }
 
 //_______________________________________________________________________
@@ -868,12 +919,12 @@ void TMVA::RuleEnsemble::Print() const
    // print function
 
    const EMsgType kmtype=kINFO;
-   const Bool_t   isDebug = (fLogger.GetMinType()<=kDEBUG);
+   const Bool_t   isDebug = (fLogger->GetMinType()<=kDEBUG);
    //
-   fLogger << kmtype << Endl;
-   fLogger << kmtype << "================================================================" << Endl;
-   fLogger << kmtype << "                          M o d e l                             " << Endl;
-   fLogger << kmtype << "================================================================" << Endl;
+   Log() << kmtype << Endl;
+   Log() << kmtype << "================================================================" << Endl;
+   Log() << kmtype << "                          M o d e l                             " << Endl;
+   Log() << kmtype << "================================================================" << Endl;
 
    Int_t ind;
    const UInt_t nvars =  GetMethodBase()->GetNvar();
@@ -881,66 +932,69 @@ void TMVA::RuleEnsemble::Print() const
    const Int_t printN = TMath::Min(10,nrules); //nrules+1;
    Int_t maxL = 0;
    for (UInt_t iv = 0; iv<fVarImportance.size(); iv++) {
-      if (GetMethodBase()->GetInputExp(iv).Length() > maxL) maxL = GetMethodBase()->GetInputExp(iv).Length();
+      if (GetMethodBase()->GetInputLabel(iv).Length() > maxL) maxL = GetMethodBase()->GetInputLabel(iv).Length();
    }
    //
    if (isDebug) {
-      fLogger << kDEBUG << "Variable importance:" << Endl;
+      Log() << kDEBUG << "Variable importance:" << Endl;
       for (UInt_t iv = 0; iv<fVarImportance.size(); iv++) {
-         fLogger << kDEBUG << setw(maxL) << GetMethodBase()->GetInputExp(iv) 
-                 << resetiosflags(ios::right) 
-                 << " : " << Form(" %3.3f",fVarImportance[iv]) << Endl;
+         Log() << kDEBUG << std::setw(maxL) << GetMethodBase()->GetInputLabel(iv) 
+               << std::resetiosflags(std::ios::right) 
+               << " : " << Form(" %3.3f",fVarImportance[iv]) << Endl;
       }
    }
    //
-   fLogger << kmtype << "Offset (a0) = " << fOffset << Endl;
+   Log() << kmtype << "Offset (a0) = " << fOffset << Endl;
    //
    if (DoLinear()) {
       if (fLinNorm.size() > 0) {
-         fLogger << kmtype << "------------------------------------" << Endl;
-         fLogger << kmtype << "Linear model (weights unnormalised)" << Endl;
-         fLogger << kmtype << "------------------------------------" << Endl;
-         fLogger << kmtype << setw(maxL) << "Variable"
-            << resetiosflags(ios::right) << " : "
-            << setw(11) << " Weights"
-            << resetiosflags(ios::right) << " : "
-            << "Importance"
-            << resetiosflags(ios::right)
-            << Endl;
-         fLogger << kmtype << "------------------------------------" << Endl;
+         Log() << kmtype << "------------------------------------" << Endl;
+         Log() << kmtype << "Linear model (weights unnormalised)" << Endl;
+         Log() << kmtype << "------------------------------------" << Endl;
+         Log() << kmtype << std::setw(maxL) << "Variable"
+               << std::resetiosflags(std::ios::right) << " : "
+               << std::setw(11) << " Weights"
+               << std::resetiosflags(std::ios::right) << " : "
+               << "Importance"
+               << std::resetiosflags(std::ios::right)
+               << Endl;
+         Log() << kmtype << "------------------------------------" << Endl;
          for ( UInt_t i=0; i<fLinNorm.size(); i++ ) {
-            fLogger << kmtype << setw(std::max(maxL,8)) << GetMethodBase()->GetInputExp(i);
+            Log() << kmtype << std::setw(std::max(maxL,8)) << GetMethodBase()->GetInputLabel(i);
             if (fLinTermOK[i]) {
-               fLogger << kmtype
-                       << resetiosflags(ios::right)
-                       << " : " << Form(" %10.3e",fLinCoefficients[i]*fLinNorm[i])
-                       << " : " << Form(" %3.3f",fLinImportance[i]/fImportanceRef) << Endl;
-            } else {
-               fLogger << kmtype << "-> importance below threshhold = "
-                       << Form(" %3.3f",fLinImportance[i]/fImportanceRef) << Endl;
+               Log() << kmtype
+                     << std::resetiosflags(std::ios::right)
+                     << " : " << Form(" %10.3e",fLinCoefficients[i]*fLinNorm[i])
+                     << " : " << Form(" %3.3f",fLinImportance[i]/fImportanceRef) << Endl;
+            } 
+            else {
+               Log() << kmtype << "-> importance below threshhold = "
+                     << Form(" %3.3f",fLinImportance[i]/fImportanceRef) << Endl;
             }
          }
-         fLogger << kmtype << "------------------------------------" << Endl;
+         Log() << kmtype << "------------------------------------" << Endl;
       }
    } 
-   else fLogger << kmtype << "Linear terms were disabled" << Endl;
+   else Log() << kmtype << "Linear terms were disabled" << Endl;
 
    if ((!DoRules()) || (nrules==0)) {
       if (!DoRules()) {
-         fLogger << kmtype << "Rule terms were disabled" << Endl;
-      } else {
-         fLogger << kmtype << "Eventhough rules were included in the model, none passed! " << nrules << Endl;
+         Log() << kmtype << "Rule terms were disabled" << Endl;
+      } 
+      else {
+         Log() << kmtype << "Eventhough rules were included in the model, none passed! " << nrules << Endl;
       }
-   } else {
-      fLogger << kmtype << "Number of rules = " << nrules << Endl;
+   } 
+   else {
+      Log() << kmtype << "Number of rules = " << nrules << Endl;
       if (isDebug) {
-         fLogger << kmtype << "N(cuts) in rules, average = " << fRuleNCave << Endl;
-         fLogger << kmtype << "                      RMS = " << fRuleNCsig << Endl;
-         fLogger << kmtype << "Fraction of signal rules = " << fRuleFSig << Endl;
-         fLogger << kmtype << "Fraction of rules containing a variable (%):" << Endl;
+         Log() << kmtype << "N(cuts) in rules, average = " << fRuleNCave << Endl;
+         Log() << kmtype << "                      RMS = " << fRuleNCsig << Endl;
+         Log() << kmtype << "Fraction of signal rules = " << fRuleFSig << Endl;
+         Log() << kmtype << "Fraction of rules containing a variable (%):" << Endl;
          for ( UInt_t v=0; v<nvars; v++) {
-            fLogger << kmtype << "   " << setw(maxL) << GetMethodBase()->GetInputExp(v);
-            fLogger << kmtype << Form(" = %2.2f",fRuleVarFrac[v]*100.0) << " %" << Endl;
+            Log() << kmtype << "   " << std::setw(maxL) << GetMethodBase()->GetInputLabel(v);
+            Log() << kmtype << Form(" = %2.2f",fRuleVarFrac[v]*100.0) << " %" << Endl;
          }
       }
       //
@@ -952,58 +1006,145 @@ void TMVA::RuleEnsemble::Print() const
       }
       sortedImp.sort();
       //
-      fLogger << kmtype << "Printing the first " << printN << " rules, ordered in importance." << Endl;
+      Log() << kmtype << "Printing the first " << printN << " rules, ordered in importance." << Endl;
       int pind=0;
       for ( std::list< std::pair<double,int> >::reverse_iterator itpair = sortedImp.rbegin();
             itpair != sortedImp.rend(); itpair++ ) {
          ind = itpair->second;
          //    if (pind==0) impref = 
-         //         fLogger << kmtype << "Rule #" << 
-         //         fLogger << kmtype << *fRules[ind] << Endl;
+         //         Log() << kmtype << "Rule #" << 
+         //         Log() << kmtype << *fRules[ind] << Endl;
          fRules[ind]->PrintLogger(Form("Rule %4d : ",pind+1));
          pind++;
          if (pind==printN) {
             if (nrules==printN) {
-               fLogger << kmtype << "All rules printed" << Endl;
-            } else {
-               fLogger << kmtype << "Skipping the next " << nrules-printN << " rules" << Endl;
+               Log() << kmtype << "All rules printed" << Endl;
+            } 
+            else {
+               Log() << kmtype << "Skipping the next " << nrules-printN << " rules" << Endl;
             }
             break;
          }
       }
    }
-   fLogger << kmtype << "================================================================" << Endl;
-   fLogger << kmtype << Endl;
+   Log() << kmtype << "================================================================" << Endl;
+   Log() << kmtype << Endl;
 }
 
 //_______________________________________________________________________
 void TMVA::RuleEnsemble::PrintRaw( ostream & os ) const
 {
    // write rules to stream
+   Int_t dp = os.precision();
    UInt_t nrules = fRules.size();
    //   std::sort(fRules.begin(),fRules.end());
    //
-   os << "ImportanceCut= "    << fImportanceCut << endl;
-   os << "LinQuantile= "      << fLinQuantile   << endl;
-   os << "AverageSupport= "   << fAverageSupport << endl;
-   os << "AverageRuleSigma= " << fAverageRuleSigma << endl;
-   os << "Offset= "           << fOffset << endl;
-   os << "NRules= "           << nrules << endl; 
+   os << "ImportanceCut= "    << fImportanceCut << std::endl;
+   os << "LinQuantile= "      << fLinQuantile   << std::endl;
+   os << "AverageSupport= "   << fAverageSupport << std::endl;
+   os << "AverageRuleSigma= " << fAverageRuleSigma << std::endl;
+   os << "Offset= "           << fOffset << std::endl;
+   os << "NRules= "           << nrules << std::endl; 
    for (UInt_t i=0; i<nrules; i++){
-      os << "***Rule " << i << endl;
+      os << "***Rule " << i << std::endl;
       (fRules[i])->PrintRaw(os);
    }
    UInt_t nlinear = fLinNorm.size();
    //
-   os << "NLinear= " << fLinTermOK.size() << endl;
+   os << "NLinear= " << fLinTermOK.size() << std::endl;
    for (UInt_t i=0; i<nlinear; i++) {
-      os << "***Linear " << i << endl;
-      os << setprecision(10) << (fLinTermOK[i] ? 1:0) << " "
+      os << "***Linear " << i << std::endl;
+      os << std::setprecision(10) << (fLinTermOK[i] ? 1:0) << " "
          << fLinCoefficients[i] << " "
          << fLinNorm[i] << " "
          << fLinDM[i] << " "
          << fLinDP[i] << " "
-         << fLinImportance[i] << " " << endl;
+         << fLinImportance[i] << " " << std::endl;
+   }
+   os << std::setprecision(dp);
+}
+
+//_______________________________________________________________________
+void* TMVA::RuleEnsemble::AddXMLTo(void* parent) const
+{
+   // write rules to XML
+   void* re = gTools().AddChild( parent, "Weights" ); // this is the "RuleEnsemble"
+
+   UInt_t nrules  = fRules.size();
+   UInt_t nlinear = fLinNorm.size();
+   gTools().AddAttr( re, "NRules",           nrules );
+   gTools().AddAttr( re, "NLinear",          nlinear );
+   gTools().AddAttr( re, "LearningModel",    (int)fLearningModel );
+   gTools().AddAttr( re, "ImportanceCut",    fImportanceCut );
+   gTools().AddAttr( re, "LinQuantile",      fLinQuantile );
+   gTools().AddAttr( re, "AverageSupport",   fAverageSupport );
+   gTools().AddAttr( re, "AverageRuleSigma", fAverageRuleSigma );
+   gTools().AddAttr( re, "Offset",           fOffset );
+   for (UInt_t i=0; i<nrules; i++) fRules[i]->AddXMLTo(re);
+
+   for (UInt_t i=0; i<nlinear; i++) {
+      void* lin = gTools().AddChild( re, "Linear" );
+      gTools().AddAttr( lin, "OK",         (fLinTermOK[i] ? 1:0) );
+      gTools().AddAttr( lin, "Coeff",      fLinCoefficients[i] );
+      gTools().AddAttr( lin, "Norm",       fLinNorm[i] );
+      gTools().AddAttr( lin, "DM",         fLinDM[i] );
+      gTools().AddAttr( lin, "DP",         fLinDP[i] );
+      gTools().AddAttr( lin, "Importance", fLinImportance[i] );
+   }
+   return re;
+}
+
+//_______________________________________________________________________
+void TMVA::RuleEnsemble::ReadFromXML( void* wghtnode ) 
+{
+   // read rules from XML
+   UInt_t nrules, nlinear;
+   gTools().ReadAttr( wghtnode, "NRules",   nrules );
+   gTools().ReadAttr( wghtnode, "NLinear",  nlinear );
+   Int_t iLearningModel;
+   gTools().ReadAttr( wghtnode, "LearningModel",     iLearningModel );
+   fLearningModel =  (ELearningModel) iLearningModel;
+   gTools().ReadAttr( wghtnode, "ImportanceCut",     fImportanceCut );
+   gTools().ReadAttr( wghtnode, "LinQuantile",       fLinQuantile );
+   gTools().ReadAttr( wghtnode, "AverageSupport",    fAverageSupport );
+   gTools().ReadAttr( wghtnode, "AverageRuleSigma",  fAverageRuleSigma );
+   gTools().ReadAttr( wghtnode, "Offset",            fOffset );
+
+   // read rules
+   DeleteRules();
+
+   UInt_t i = 0;
+   fRules.resize( nrules  );
+   void* ch = gTools().GetChild( wghtnode );
+   for (i=0; i<nrules; i++) {
+      fRules[i] = new Rule();
+      fRules[i]->SetRuleEnsemble( this );
+      fRules[i]->ReadFromXML( ch );
+
+      ch = gTools().GetNextChild(ch);
+   }
+
+   // read linear classifier (Fisher)
+   fLinNorm        .resize( nlinear );
+   fLinTermOK      .resize( nlinear );
+   fLinCoefficients.resize( nlinear );
+   fLinDP          .resize( nlinear );
+   fLinDM          .resize( nlinear );
+   fLinImportance  .resize( nlinear );
+
+   Int_t iok;
+   i=0;
+   while(ch) {
+      gTools().ReadAttr( ch, "OK",         iok );
+      fLinTermOK[i] = (iok == 1);
+      gTools().ReadAttr( ch, "Coeff",      fLinCoefficients[i]  );
+      gTools().ReadAttr( ch, "Norm",       fLinNorm[i]          );
+      gTools().ReadAttr( ch, "DM",         fLinDM[i]            );
+      gTools().ReadAttr( ch, "DP",         fLinDP[i]            );
+      gTools().ReadAttr( ch, "Importance", fLinImportance[i]    );
+
+      i++;
+      ch = gTools().GetNextChild(ch);
    }
 }
 
@@ -1013,9 +1154,8 @@ void TMVA::RuleEnsemble::ReadRaw( istream & istr )
    // read rule ensemble from stream
    UInt_t nrules;
    //
-   string dummy;
+   std::string dummy;
    Int_t idum;
-   //   Double_t ddum;
    //
    // First block is general stuff
    //
@@ -1041,14 +1181,15 @@ void TMVA::RuleEnsemble::ReadRaw( istream & istr )
    //
    UInt_t nlinear;
    //
+   // coverity[tainted_data_argument]
    istr >> dummy >> nlinear;
    //
-   fLinTermOK.resize(nlinear);
-   fLinCoefficients.resize(nlinear);
-   fLinNorm.resize(nlinear);
-   fLinDP.resize(nlinear);
-   fLinDM.resize(nlinear);
-   fLinImportance.resize(nlinear);
+   fLinNorm        .resize( nlinear );
+   fLinTermOK      .resize( nlinear );
+   fLinCoefficients.resize( nlinear );
+   fLinDP          .resize( nlinear );
+   fLinDM          .resize( nlinear );
+   fLinImportance  .resize( nlinear );
    //
 
    Int_t iok;
@@ -1077,6 +1218,16 @@ void TMVA::RuleEnsemble::Copy( const RuleEnsemble & other )
       fVarImportance     = other.GetVarImportance();
       fLearningModel     = other.GetLearningModel();
       fLinQuantile       = other.GetLinQuantile();
+      fRuleNCsig         = other.fRuleNCsig;
+      fAverageRuleSigma  = other.fAverageRuleSigma;
+      fEventCacheOK      = other.fEventCacheOK;
+      fImportanceRef     = other.fImportanceRef;
+      fNRulesGenerated   = other.fNRulesGenerated;
+      fRuleFSig          = other.fRuleFSig;
+      fRuleMapInd0       = other.fRuleMapInd0;
+      fRuleMapInd1       = other.fRuleMapInd1;
+      fRuleMapOK         = other.fRuleMapOK;
+      fRuleNCave         = other.fRuleNCave;
    }
 }
 
@@ -1097,7 +1248,6 @@ void TMVA::RuleEnsemble::FindNEndNodes( const Node *node, Int_t & nendnodes )
    // find the number of leaf nodes
 
    if (node==0) return;
-   //   if (dynamic_cast<const DecisionTreeNode*>(node)->GetSelector()<0) {
    if ((node->GetRight()==0) && (node->GetLeft()==0)) {
       ++nendnodes;
       return;
@@ -1134,7 +1284,7 @@ void TMVA::RuleEnsemble::AddRule( const Node *node )
          AddRule( node->GetLeft() );
       } 
       else {
-         fLogger << kFATAL << "<AddRule> - ERROR failed in creating a rule! BUG!" << Endl;
+         Log() << kFATAL << "<AddRule> - ERROR failed in creating a rule! BUG!" << Endl;
       }
    }
 }
@@ -1150,7 +1300,7 @@ TMVA::Rule *TMVA::RuleEnsemble::MakeTheRule( const Node *node )
    //
    //
    if (node==0) {
-      fLogger << kFATAL << "<MakeTheRule> Input node is NULL. Should not happen. BUG!" << Endl;
+      Log() << kFATAL << "<MakeTheRule> Input node is NULL. Should not happen. BUG!" << Endl;
       return 0;
    }
 
@@ -1167,17 +1317,18 @@ TMVA::Rule *TMVA::RuleEnsemble::MakeTheRule( const Node *node )
    nodeVec.push_back( node );
    while (parent!=0) {
       parent = parent->GetParent();
-      if (parent) {
-         if (dynamic_cast<const DecisionTreeNode*>(parent)->GetSelector()>=0)
-            nodeVec.insert( nodeVec.begin(), parent );
-      }
+      if (!parent) continue;
+      const DecisionTreeNode* dtn = dynamic_cast<const DecisionTreeNode*>(parent);
+      if (dtn && dtn->GetSelector()>=0)
+         nodeVec.insert( nodeVec.begin(), parent );
+
    }
    if (nodeVec.size()<2) {
-      fLogger << kFATAL << "<MakeTheRule> BUG! Inconsistent Rule!" << Endl;
+      Log() << kFATAL << "<MakeTheRule> BUG! Inconsistent Rule!" << Endl;
       return 0;
    }
    Rule *rule = new Rule( this, nodeVec );
-   rule->SetMsgType( fLogger.GetMinType() );
+   rule->SetMsgType( Log().GetMinType() );
    return rule;
 }
 
@@ -1186,7 +1337,7 @@ void TMVA::RuleEnsemble::MakeRuleMap(const std::vector<Event *> *events, UInt_t
 {
    // Makes rule map for all events
 
-   fLogger << kVERBOSE << "Making Rule map for all events" << Endl;
+   Log() << kVERBOSE << "Making Rule map for all events" << Endl;
    // make rule response map
    if (events==0) events = GetTrainingEvents();
    if ((ifirst==0) || (ilast==0) || (ifirst>ilast)) {
@@ -1201,7 +1352,7 @@ void TMVA::RuleEnsemble::MakeRuleMap(const std::vector<Event *> *events, UInt_t
    }
    //
    if (fRuleMapOK) {
-      fLogger << kVERBOSE << "<MakeRuleMap> Map is already valid" << Endl;
+      Log() << kVERBOSE << "<MakeRuleMap> Map is already valid" << Endl;
       return;  // already cached
    }
    fRuleMapEvents = events;
@@ -1210,7 +1361,7 @@ void TMVA::RuleEnsemble::MakeRuleMap(const std::vector<Event *> *events, UInt_t
    // check number of rules
    UInt_t nrules = GetNRules(); 
    if (nrules==0) {
-      fLogger << kVERBOSE << "No rules found in MakeRuleMap()" << Endl;
+      Log() << kVERBOSE << "No rules found in MakeRuleMap()" << Endl;
       fRuleMapOK = kTRUE;
       return;
    }
@@ -1229,7 +1380,7 @@ void TMVA::RuleEnsemble::MakeRuleMap(const std::vector<Event *> *events, UInt_t
       }
    }
    fRuleMapOK = kTRUE;
-   fLogger << kVERBOSE << "Made rule map for event# " << ifirst << " : " << ilast << Endl;
+   Log() << kVERBOSE << "Made rule map for event# " << ifirst << " : " << ilast << Endl;
 }
 
 //_______________________________________________________________________
