@@ -1,7 +1,9 @@
 //////////////////////////////////////////////////////////
 //
 // Example of TSelector implementation to do generic
-// processing (filling a set of histograms in this case)
+// processing (filling a set of histograms in this case).
+// See tutorials/proof/runProof.C, option "simple", for an
+// example of how to run this selector.
 //
 //////////////////////////////////////////////////////////
 
@@ -11,6 +13,7 @@
 #include <TSelector.h>
 
 class TH1F;
+class TH3F;
 class TRandom3;
 
 class ProofSimple : public TSelector {
@@ -18,8 +21,10 @@ public :
 
    // Specific members
    Int_t            fNhist;
-   TH1F           **fHist;//[fNhist]
-   TRandom3        *fRandom;
+   TH1F           **fHist;//![fNhist]
+   Int_t            fNhist3;
+   TH3F           **fHist3;//![fNhist3]
+   TRandom3        *fRandom;//!
 
    ProofSimple();
    virtual ~ProofSimple();
@@ -33,8 +38,14 @@ public :
    virtual TList  *GetOutputList() const { return fOutput; }
    virtual void    SlaveTerminate();
    virtual void    Terminate();
+   
+   // Setters and getters (for TDataMember)
+   Int_t GetNhist() { return fNhist; }
+   void SetNhist(Int_t nh) { fNhist = nh; }
+   Int_t GetNhist3() { return fNhist3; }
+   void SetNhist3(Int_t nh) { fNhist3 = nh; }
 
-   ClassDef(ProofSimple,0);
+   ClassDef(ProofSimple,2);
 };
 
 #endif
